@@ -14,6 +14,7 @@
 
 #include "divemode.h"
 #include "equipment.h"
+#include "picture.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -128,7 +129,6 @@ struct divecomputer {
 	struct divecomputer *next;
 };
 
-struct picture;
 struct dive_site;
 struct dive_site_table;
 struct dive_table;
@@ -160,7 +160,7 @@ struct dive {
 	struct tag_entry *tag_list;
 	struct divecomputer dc;
 	int id; // unique ID for this dive
-	struct picture *picture_list;
+	struct picture_table pictures;
 	unsigned char git_id[20];
 	bool notrip; /* Don't autogroup this dive to a trip */
 	bool selected;
@@ -204,15 +204,6 @@ extern enum divemode_t get_current_divemode(const struct divecomputer *dc, int t
 extern struct event *get_next_divemodechange(const struct event **evd, bool update_pointer);
 extern enum divemode_t get_divemode_at_time(const struct divecomputer *dc, int dtime, const struct event **ev_dmc);
 
-/* picture list and methods related to dive picture handling */
-#define FOR_EACH_PICTURE(_dive) \
-	if (_dive)              \
-		for (struct picture *picture = (_dive)->picture_list; picture; picture = picture->next)
-extern void create_picture(const char *filename, int shift_time, bool match_all);
-extern void dive_add_picture(struct dive *d, struct picture *newpic);
-extern bool dive_remove_picture(struct dive *d, const char *filename);
-extern bool picture_check_valid_time(timestamp_t timestamp, int shift_time);
-
 extern bool has_gaschange_event(const struct dive *dive, const struct divecomputer *dc, int idx);
 extern int explicit_first_cylinder(const struct dive *dive, const struct divecomputer *dc);
 extern int get_depth_at_time(const struct divecomputer *dc, unsigned int time);
@@ -281,9 +272,7 @@ extern bool dive_site_has_gps_location(const struct dive_site *ds);
 extern int dive_has_gps_location(const struct dive *dive);
 extern location_t dive_get_gps_location(const struct dive *d);
 
-extern bool dive_within_time_range(struct dive *dive, timestamp_t when, timestamp_t offset);
-extern bool time_during_dive_with_offset(struct dive *dive, timestamp_t when, timestamp_t offset);
-struct dive *find_dive_n_near(timestamp_t when, int n, timestamp_t offset);
+extern bool time_during_dive_with_offset(const struct dive *dive, timestamp_t when, timestamp_t offset);
 
 /* Check if two dive computer entries are the exact same dive (-1=no/0=maybe/1=yes) */
 extern int match_one_dc(const struct divecomputer *a, const struct divecomputer *b);
@@ -332,6 +321,7 @@ extern void sort_dive_table(struct dive_table *table);
 extern struct dive *fixup_dive(struct dive *dive);
 extern pressure_t calculate_surface_pressure(const struct dive *dive);
 extern pressure_t un_fixup_surface_pressure(const struct dive *d);
+extern int get_dive_salinity(const struct dive *dive);
 extern void fixup_dc_duration(struct divecomputer *dc);
 extern int dive_getUniqID();
 extern unsigned int dc_airtemp(const struct divecomputer *dc);
@@ -344,7 +334,6 @@ extern struct event *clone_event(const struct event *src_ev);
 extern void copy_events(const struct divecomputer *s, struct divecomputer *d);
 extern void copy_events_until(const struct dive *sd, struct dive *dd, int time);
 extern void free_events(struct event *ev);
-extern void copy_cylinders(const struct cylinder_table *s, struct cylinder_table *d);
 extern void copy_used_cylinders(const struct dive *s, struct dive *d, bool used_only);
 extern void copy_samples(const struct divecomputer *s, struct divecomputer *d);
 extern bool is_cylinder_used(const struct dive *dive, int idx);
