@@ -85,6 +85,11 @@ extern "C" void create_device_node(const char *model, uint32_t deviceid, const c
 	dcList.addDC(model, deviceid, nickname, serial, firmware);
 }
 
+extern "C" void clear_device_nodes()
+{
+	dcList.dcs.clear();
+}
+
 static bool compareDCById(const DiveComputerNode &a, const DiveComputerNode &b)
 {
 	return a.deviceId < b.deviceId;
@@ -117,8 +122,8 @@ extern "C" void call_for_each_dc (void *f, void (*callback)(void *, const char *
 			found = true;
 		}
 		if (found)
-	callback(f, qPrintable(node.model), node.deviceId, qPrintable(node.nickName),
-				 qPrintable(node.serialNumber), qPrintable(node.firmware));
+			callback(f, qPrintable(node.model), node.deviceId, qPrintable(node.nickName),
+						 qPrintable(node.serialNumber), qPrintable(node.firmware));
 	}
 }
 
@@ -158,3 +163,13 @@ extern "C" void set_dc_nickname(struct dive *dive)
 		}
 	}
 }
+
+QString get_dc_nickname(const struct divecomputer *dc)
+{
+	const DiveComputerNode *existNode = dcList.getExact(dc->model, dc->deviceid);
+
+	if (existNode && !existNode->nickName.isEmpty())
+		return existNode->nickName;
+	else
+		return dc->model;
+}
