@@ -177,7 +177,7 @@ static int reset_to_remote(git_repository *repo, git_reference *local, const git
 	git_object *target;
 
 	if (verbose)
-		fprintf(stderr, "git storage: reset to remote\n");
+		SSRF_INFO("git storage: reset to remote\n");
 
 	// If it's not checked out (bare or not HEAD), just update the reference */
 	if (git_repository_is_bare(repo) || git_branch_is_head(local) != 1) {
@@ -286,24 +286,18 @@ int credential_https_cb(git_cred **out,
 	return git_cred_userpass_plaintext_new(out, username, password);
 }
 
-#define KNOWN_CERT "\xfd\xb8\xf7\x73\x76\xe2\x75\x53\x93\x37\xdc\xfe\x1e\x55\x43\x3d\xf2\x2c\x18\x2c"
 int certificate_check_cb(git_cert *cert, int valid, const char *host, void *payload)
 {
 	UNUSED(payload);
+	if (verbose)
+		SSRF_INFO("certificate callback for host %s with validity %d\n", host, valid);
 	if (same_string(host, "cloud.subsurface-divelog.org") && cert->cert_type == GIT_CERT_X509) {
-		SHA_CTX ctx;
-		unsigned char hash[21];
-		git_cert_x509 *cert509 = (git_cert_x509 *)cert;
-		SHA1_Init(&ctx);
-		SHA1_Update(&ctx, cert509->data, cert509->len);
-		SHA1_Final(hash, &ctx);
-		hash[20] = 0;
-		if (verbose > 1)
-			if (same_string((char *)hash, KNOWN_CERT)) {
-				fprintf(stderr, "cloud certificate considered %s, forcing it valid\n",
-					valid ? "valid" : "not valid");
-				return 1;
-			}
+		// for some reason the LetsEncrypt certificate makes libgit2 throw up on some
+		// platforms but not on others
+		// if we are connecting to the cloud server we alrady called 'canReachCloudServer()'
+		// which will fail if the SSL certificate isn't valid, so let's simply always
+		// tell the caller that this certificate is valid
+		return 1;
 	}
 	return valid;
 }
@@ -318,7 +312,7 @@ static int update_remote(git_repository *repo, git_remote *origin, git_reference
 	const char *name = git_reference_name(local);
 
 	if (verbose)
-		fprintf(stderr, "git storage: update remote\n");
+		SSRF_INFO("git storage: update remote\n");
 
 	refspec.count = 1;
 	refspec.strings = (char **)&name;
@@ -349,49 +343,46 @@ static int try_to_git_merge(git_repository *repo, git_reference **local_p, git_r
 	git_commit *local_commit, *remote_commit, *base_commit;
 	git_index *merged_index;
 	git_merge_options merge_options;
+	struct membuffer msg = { 0, 0, NULL};
 
 	if (verbose) {
 		char outlocal[41], outremote[41];
 		outlocal[40] = outremote[40] = 0;
 		git_oid_fmt(outlocal, local_id);
 		git_oid_fmt(outremote, remote_id);
-		fprintf(stderr, "trying to merge local SHA %s remote SHA %s\n", outlocal, outremote);
+		SSRF_INFO("trying to merge local SHA %s remote SHA %s\n", outlocal, outremote);
 	}
 
 	git_merge_init_options(&merge_options, GIT_MERGE_OPTIONS_VERSION);
-#if !LIBGIT2_VER_MAJOR && LIBGIT2_VER_MINOR > 23
 	merge_options.flags = GIT_MERGE_FIND_RENAMES;
-#else
-	merge_options.tree_flags = GIT_MERGE_TREE_FIND_RENAMES;
-#endif
 	merge_options.file_favor = GIT_MERGE_FILE_FAVOR_UNION;
 	merge_options.rename_threshold = 100;
 	if (git_commit_lookup(&local_commit, repo, local_id)) {
-		fprintf(stderr, "Remote storage and local data diverged. Error: can't get commit (%s)", giterr_last()->message);
+		SSRF_INFO("Remote storage and local data diverged. Error: can't get commit (%s)", giterr_last()->message);
 		goto diverged_error;
 	}
 	if (git_commit_tree(&local_tree, local_commit)) {
-		fprintf(stderr, "Remote storage and local data diverged. Error: failed local tree lookup (%s)", giterr_last()->message);
+		SSRF_INFO("Remote storage and local data diverged. Error: failed local tree lookup (%s)", giterr_last()->message);
 		goto diverged_error;
 	}
 	if (git_commit_lookup(&remote_commit, repo, remote_id)) {
-		fprintf(stderr, "Remote storage and local data diverged. Error: can't get commit (%s)", giterr_last()->message);
+		SSRF_INFO("Remote storage and local data diverged. Error: can't get commit (%s)", giterr_last()->message);
 		goto diverged_error;
 	}
 	if (git_commit_tree(&remote_tree, remote_commit)) {
-		fprintf(stderr, "Remote storage and local data diverged. Error: failed local tree lookup (%s)", giterr_last()->message);
+		SSRF_INFO("Remote storage and local data diverged. Error: failed local tree lookup (%s)", giterr_last()->message);
 		goto diverged_error;
 	}
 	if (git_commit_lookup(&base_commit, repo, base)) {
-		fprintf(stderr, "Remote storage and local data diverged. Error: can't get commit (%s)", giterr_last()->message);
+		SSRF_INFO("Remote storage and local data diverged. Error: can't get commit (%s)", giterr_last()->message);
 		goto diverged_error;
 	}
 	if (git_commit_tree(&base_tree, base_commit)) {
-		fprintf(stderr, "Remote storage and local data diverged. Error: failed base tree lookup (%s)", giterr_last()->message);
+		SSRF_INFO("Remote storage and local data diverged. Error: failed base tree lookup (%s)", giterr_last()->message);
 		goto diverged_error;
 	}
 	if (git_merge_trees(&merged_index, repo, base_tree, local_tree, remote_tree, &merge_options)) {
-		fprintf(stderr, "Remote storage and local data diverged. Error: merge failed (%s)", giterr_last()->message);
+		SSRF_INFO("Remote storage and local data diverged. Error: merge failed (%s)", giterr_last()->message);
 		// this is the one where I want to report more detail to the user - can't quite explain why
 		return report_error(translate("gettextFromC", "Remote storage and local data diverged. Error: merge failed (%s)"), giterr_last()->message);
 	}
@@ -405,20 +396,20 @@ static int try_to_git_merge(git_repository *repo, git_reference **local_p, git_r
 		while (git_index_conflict_next(&ancestor, &ours, &theirs, iter)
 		       != GIT_ITEROVER) {
 			/* Mark this conflict as resolved */
-			fprintf(stderr, "conflict in %s / %s / %s -- ",
+			SSRF_INFO("conflict in %s / %s / %s -- ",
 				ours ? ours->path : "-",
 				theirs ? theirs->path : "-",
 				ancestor ? ancestor->path : "-");
 			if ((!ours && theirs && ancestor) ||
 			    (ours && !theirs && ancestor)) {
 				// the file was removed on one side or the other - just remove it
-				fprintf(stderr, "looks like a delete on one side; removing the file from the index\n");
+				SSRF_INFO("looks like a delete on one side; removing the file from the index\n");
 				error = git_index_remove(merged_index, ours ? ours->path : theirs->path, GIT_INDEX_STAGE_ANY);
 			} else if (ancestor) {
 				error = git_index_conflict_remove(merged_index, ours ? ours->path : theirs ? theirs->path : ancestor->path);
 			}
 			if (error) {
-				fprintf(stderr, "error at conflict resplution (%s)", giterr_last()->message);
+				SSRF_INFO("error at conflict resplution (%s)", giterr_last()->message);
 			}
 		}
 		git_index_conflict_cleanup(merged_index);
@@ -434,10 +425,12 @@ static int try_to_git_merge(git_repository *repo, git_reference **local_p, git_r
 		goto write_error;
 	if (git_tree_lookup(&merged_tree, repo, &merge_oid))
 		goto write_error;
-	if (git_signature_default(&author, repo) < 0)
-		if (git_signature_now(&author, "Subsurface", "noemail@given") < 0)
-			goto write_error;
-	if (git_commit_create_v(&commit_oid, repo, NULL, author, author, NULL, "automatic merge", merged_tree, 2, local_commit, remote_commit))
+	if (get_authorship(repo, &author) < 0)
+		goto write_error;
+	const char *user_agent = subsurface_user_agent();
+	put_format(&msg, "Automatic merge\n\nCreated by %s\n", user_agent);
+	free((void *)user_agent);
+	if (git_commit_create_v(&commit_oid, repo, NULL, author, author, NULL, mb_cstring(&msg), merged_tree, 2, local_commit, remote_commit))
 		goto write_error;
 	if (git_commit_lookup(&commit, repo, &commit_oid))
 		goto write_error;
@@ -453,13 +446,15 @@ static int try_to_git_merge(git_repository *repo, git_reference **local_p, git_r
 	set_git_id(&commit_oid);
 	git_signature_free(author);
 	if (verbose)
-		fprintf(stderr, "Successfully merged repositories");
+		SSRF_INFO("Successfully merged repositories");
+	free_buffer(&msg);
 	return 0;
 
 diverged_error:
 	return report_error(translate("gettextFromC", "Remote storage and local data diverged"));
 
 write_error:
+	free_buffer(&msg);
 	return report_error(translate("gettextFromC", "Remote storage and local data diverged. Error: writing the data failed (%s)"), giterr_last()->message);
 }
 
@@ -482,7 +477,7 @@ static int try_to_update(git_repository *repo, git_remote *origin, git_reference
 	int ret = 0;
 
 	if (verbose)
-		fprintf(stderr, "git storage: try to update\n");
+		SSRF_INFO("git storage: try to update\n");
 
 	if (!git_reference_cmp(local, remote))
 		return 0;
@@ -524,7 +519,7 @@ static int try_to_update(git_repository *repo, git_remote *origin, git_reference
 	/* Is the local repo the more recent one? See if we can update upstream */
 	if (git_oid_equal(&base, remote_id)) {
 		if (verbose)
-			fprintf(stderr, "local is newer than remote, update remote\n");
+			SSRF_INFO("local is newer than remote, update remote\n");
 		git_storage_update_progress(translate("gettextFromC", "Push local changes to cloud storage"));
 		return update_remote(repo, origin, local, remote, rt);
 	}
@@ -564,7 +559,7 @@ static int check_remote_status(git_repository *repo, git_remote *origin, const c
 	git_reference *local_ref, *remote_ref;
 
 	if (verbose)
-		fprintf(stderr, "git storage: check remote status\n");
+		SSRF_INFO("git storage: check remote status\n");
 
 	if (git_branch_lookup(&local_ref, repo, branch, GIT_BRANCH_LOCAL)) {
 		if (is_subsurface_cloud)
@@ -595,6 +590,57 @@ static int check_remote_status(git_repository *repo, git_remote *origin, const c
 	return error;
 }
 
+/* this is (so far) only used by the git storage tests to remove a remote branch
+ * it will print out errors, but not return an error (as this isn't a function that
+ * we test as part of the tests, it's a helper to not leave loads of dead branches on
+ * the server)
+ */
+void delete_remote_branch(git_repository *repo, const char *remote, const char *branch)
+{
+	int error;
+	char *proxy_string;
+	git_remote *origin;
+	git_config *conf;
+
+	/* set up the config and proxy information in order to connect to the server */
+	git_repository_config(&conf, repo);
+	if (getProxyString(&proxy_string)) {
+		git_config_set_string(conf, "http.proxy", proxy_string);
+		free(proxy_string);
+	} else {
+		git_config_delete_entry(conf, "http.proxy");
+	}
+	if (git_remote_lookup(&origin, repo, "origin")) {
+		SSRF_INFO("Repository '%s' origin lookup failed (%s)", remote, giterr_last() ? giterr_last()->message : "(unspecified)");
+		return;
+	}
+	/* fetch the remote state */
+	git_fetch_options f_opts = GIT_FETCH_OPTIONS_INIT;
+	auth_attempt = 0;
+	f_opts.callbacks.credentials = credential_https_cb;
+	error = git_remote_fetch(origin, NULL, &f_opts, NULL);
+	if (error) {
+		SSRF_INFO("remote fetch failed (%s)\n", giterr_last() ? giterr_last()->message : "authentication failed");
+		return;
+	}
+	/* delete the remote branch by pushing to ":refs/heads/<branch>" */
+	git_strarray refspec;
+	char *branch_ref = format_string(":refs/heads/%s", branch);
+	refspec.count = 1;
+	refspec.strings = &branch_ref;
+	git_push_options p_opts = GIT_PUSH_OPTIONS_INIT;
+	auth_attempt = 0;
+	p_opts.callbacks.credentials = credential_https_cb;
+	error = git_remote_push(origin, &refspec, &p_opts);
+	free(branch_ref);
+	if (error) {
+		SSRF_INFO("Unable to delete branch '%s'", branch);
+		SSRF_INFO("error was (%s)\n", giterr_last() ? giterr_last()->message : "(unspecified)");
+	}
+	git_remote_free(origin);
+	return;
+}
+
 int sync_with_remote(git_repository *repo, const char *remote, const char *branch, enum remote_transport rt)
 {
 	int error;
@@ -604,21 +650,21 @@ int sync_with_remote(git_repository *repo, const char *remote, const char *branc
 
 	if (git_local_only) {
 		if (verbose)
-			fprintf(stderr, "don't sync with remote - read from cache only\n");
+			SSRF_INFO("don't sync with remote - read from cache only\n");
 		return 0;
 	}
 	if (verbose)
-		fprintf(stderr, "sync with remote %s[%s]\n", remote, branch);
+		SSRF_INFO("sync with remote %s[%s]\n", remote, branch);
 	git_storage_update_progress(translate("gettextFromC", "Sync with cloud storage"));
 	git_repository_config(&conf, repo);
 	if (rt == RT_HTTPS && getProxyString(&proxy_string)) {
 		if (verbose)
-			fprintf(stderr, "set proxy to \"%s\"\n", proxy_string);
+			SSRF_INFO("set proxy to \"%s\"\n", proxy_string);
 		git_config_set_string(conf, "http.proxy", proxy_string);
 		free(proxy_string);
 	} else {
 		if (verbose)
-			fprintf(stderr, "delete proxy setting\n");
+			SSRF_INFO("delete proxy setting\n");
 		git_config_delete_entry(conf, "http.proxy");
 	}
 
@@ -640,7 +686,7 @@ int sync_with_remote(git_repository *repo, const char *remote, const char *branc
 		return 0;
 	}
 	if (verbose)
-		fprintf(stderr, "git storage: fetch remote\n");
+		SSRF_INFO("git storage: fetch remote\n");
 	git_fetch_options opts = GIT_FETCH_OPTIONS_INIT;
 	opts.callbacks.transfer_progress = &transfer_progress_cb;
 	auth_attempt = 0;
@@ -659,7 +705,7 @@ int sync_with_remote(git_repository *repo, const char *remote, const char *branc
 			report_error("Unable to fetch remote '%s'", remote);
 		if (verbose)
 			// If we returned GIT_EUSER during authentication, giterr_last() returns NULL
-			fprintf(stderr, "remote fetch failed (%s)\n",
+			SSRF_INFO("remote fetch failed (%s)\n",
 				giterr_last() ? giterr_last()->message : "authentication failed");
 		// Since we failed to sync with online repository, enter offline mode
 		git_local_only = true;
@@ -676,9 +722,10 @@ static git_repository *update_local_repo(const char *localdir, const char *remot
 {
 	int error;
 	git_repository *repo = NULL;
+	git_reference *head;
 
 	if (verbose)
-		fprintf(stderr, "git storage: update local repo\n");
+		SSRF_INFO("git storage: update local repo\n");
 
 	error = git_repository_open(&repo, localdir);
 	if (error) {
@@ -688,6 +735,21 @@ static git_repository *update_local_repo(const char *localdir, const char *remot
 			report_error("Unable to open git cache repository at %s: %s", localdir, giterr_last()->message);
 		return NULL;
 	}
+
+	/* Check the HEAD being the right branch */
+	if (!git_repository_head(&head, repo)) {
+		const char *name;
+		if (!git_branch_name(&name, head)) {
+			if (strcmp(name, branch)) {
+				char *branchref = format_string("refs/heads/%s", branch);
+				SSRF_INFO("Setting cache branch from '%s' to '%s'", name, branch);
+				git_repository_set_head(repo, branchref);
+				free(branchref);
+			}
+		}
+		git_reference_free(head);
+	}
+
 	if (!git_local_only)
 		sync_with_remote(repo, remote, branch, rt);
 
@@ -703,19 +765,19 @@ static int repository_create_cb(git_repository **out, const char *path, int bare
 	int ret = git_repository_init(out, path, bare);
 	if (ret != 0) {
 		if (verbose)
-			fprintf(stderr, "Initializing git repository failed\n");
+			SSRF_INFO("Initializing git repository failed\n");
 		return ret;
 	}
 
 	git_repository_config(&conf, *out);
 	if (getProxyString(&proxy_string)) {
 		if (verbose)
-			fprintf(stderr, "set proxy to \"%s\"\n", proxy_string);
+			SSRF_INFO("set proxy to \"%s\"\n", proxy_string);
 		git_config_set_string(conf, "http.proxy", proxy_string);
 		free(proxy_string);
 	} else {
 		if (verbose)
-			fprintf(stderr, "delete proxy setting\n");
+			SSRF_INFO("delete proxy setting\n");
 		git_config_delete_entry(conf, "http.proxy");
 	}
 	return ret;
@@ -727,17 +789,20 @@ static git_repository *create_and_push_remote(const char *localdir, const char *
 {
 	git_repository *repo;
 	git_config *conf;
-	char *variable_name, *merge_head;
+	char *variable_name, *head;
 
 	if (verbose)
-		fprintf(stderr, "git storage: create and push remote\n");
+		SSRF_INFO("git storage: create and push remote\n");
 
 	/* first make sure the directory for the local cache exists */
 	subsurface_mkdir(localdir);
 
+	head = format_string("refs/heads/%s", branch);
+
 	/* set up the origin to point to our remote */
 	git_repository_init_options init_opts = GIT_REPOSITORY_INIT_OPTIONS_INIT;
 	init_opts.origin_url = remote;
+	init_opts.initial_head = head;
 
 	/* now initialize the repository with */
 	git_repository_init_ext(&repo, localdir, &init_opts);
@@ -749,9 +814,8 @@ static git_repository *create_and_push_remote(const char *localdir, const char *
 	free(variable_name);
 
 	variable_name = format_string("branch.%s.merge", branch);
-	merge_head = format_string("refs/heads/%s", branch);
-	git_config_set_string(conf, variable_name, merge_head);
-	free(merge_head);
+	git_config_set_string(conf, variable_name, head);
+	free(head);
 	free(variable_name);
 
 	/* finally create an empty commit and push it to the remote */
@@ -767,7 +831,7 @@ static git_repository *create_local_repo(const char *localdir, const char *remot
 	git_clone_options opts = GIT_CLONE_OPTIONS_INIT;
 
 	if (verbose)
-		fprintf(stderr, "git storage: create_local_repo\n");
+		SSRF_INFO("git storage: create_local_repo\n");
 
 	auth_attempt = 0;
 	opts.fetch_opts.callbacks.transfer_progress = &transfer_progress_cb;
@@ -782,15 +846,15 @@ static git_repository *create_local_repo(const char *localdir, const char *remot
 	if (is_subsurface_cloud && !canReachCloudServer())
 		return 0;
 	if (verbose > 1)
-		fprintf(stderr, "git storage: calling git_clone()\n");
+		SSRF_INFO("git storage: calling git_clone()\n");
 	error = git_clone(&cloned_repo, remote, localdir, &opts);
 	if (verbose > 1)
-		fprintf(stderr, "git storage: returned from git_clone() with error %d\n", error);
+		SSRF_INFO("git storage: returned from git_clone() with error %d\n", error);
 	if (error) {
 		char *msg = "";
 		if (giterr_last()) {
 			 msg = giterr_last()->message;
-			 fprintf(stderr, "error message was %s\n", msg);
+			 SSRF_INFO("error message was %s\n", msg);
 		}
 		char *pattern = format_string("reference 'refs/remotes/origin/%s' not found", branch);
 		// it seems that we sometimes get 'Reference' and sometimes 'reference'
@@ -799,7 +863,7 @@ static git_repository *create_local_repo(const char *localdir, const char *remot
 			 * to our cloud storage and the branch doesn't exist.
 			 * So we need to create the branch and push it to the remote */
 			if (verbose)
-				fprintf(stderr, "remote repo didn't include our branch\n");
+				SSRF_INFO("remote repo didn't include our branch\n");
 			cloned_repo = create_and_push_remote(localdir, remote, branch);
 #if !defined(DEBUG) && !defined(SUBSURFACE_MOBILE)
 		} else if (is_subsurface_cloud) {
@@ -830,7 +894,7 @@ static struct git_repository *get_remote_repo(const char *localdir, const char *
 	enum remote_transport rt = url_to_remote_transport(remote);
 
 	if (verbose > 1) {
-		fprintf(stderr, "git_remote_repo: accessing %s\n", remote);
+		SSRF_INFO("git_remote_repo: accessing %s\n", remote);
 	}
 	git_storage_update_progress(translate("gettextFromC", "Synchronising data file"));
 	/* Do we already have a local cache? */
@@ -1013,7 +1077,7 @@ struct git_repository *is_git_repository(const char *filename, const char **bran
 
 	if (subsurface_stat(loc, &st) < 0 || !S_ISDIR(st.st_mode)) {
 		if (verbose)
-			fprintf(stderr, "loc %s wasn't found or is not a directory\n", loc);
+			SSRF_INFO("loc %s wasn't found or is not a directory\n", loc);
 		free(loc);
 		free(branch);
 		return dummy_git_repository;
