@@ -20,6 +20,12 @@ struct gpsTracker {
 	int idx;
 };
 
+struct DiveAndLocation {
+	struct dive *d;
+	location_t location;
+	QString name;
+};
+
 class GpsLocation : public QObject {
 	Q_OBJECT
 public:
@@ -27,7 +33,7 @@ public:
 	~GpsLocation();
 	static GpsLocation *instance();
 	static bool hasInstance();
-	bool applyLocations();
+	std::vector<DiveAndLocation> getLocations();
 	int getGpsNum() const;
 	bool hasLocationsSource();
 	QString currentPosition();
