@@ -18,6 +18,7 @@ static int shearwater_cylinders(void *param, int columns, char **data, char **co
 	UNUSED(columns);
 	UNUSED(column);
 	struct parser_state *state = (struct parser_state *)param;
+	cylinder_t *cyl;
 
 	int o2 = lrint(strtod_flags(data[0], NULL, 0) * 1000);
 	int he = lrint(strtod_flags(data[1], NULL, 0) * 1000);
@@ -27,9 +28,9 @@ static int shearwater_cylinders(void *param, int columns, char **data, char **co
 	if (o2 == 990 && he == 0)
 		o2 = 1000;
 
-	cylinder_start(state);
-	state->cur_dive->cylinder[state->cur_cylinder_index].gasmix.o2.permille = o2;
-	state->cur_dive->cylinder[state->cur_cylinder_index].gasmix.he.permille = he;
+	cyl = cylinder_start(state);
+	cyl->gasmix.o2.permille = o2;
+	cyl->gasmix.he.permille = he;
 	cylinder_end(state);
 
 	return 0;
@@ -40,6 +41,7 @@ static int shearwater_changes(void *param, int columns, char **data, char **colu
 	UNUSED(columns);
 	UNUSED(column);
 	struct parser_state *state = (struct parser_state *)param;
+	cylinder_t *cyl;
 
 	if (columns != 3) {
 		return 1;
@@ -58,22 +60,22 @@ static int shearwater_changes(void *param, int columns, char **data, char **colu
 	// Find the cylinder index
 	int i;
 	bool found = false;
-	for (i = 0; i < state->cur_cylinder_index; ++i) {
-		if (state->cur_dive->cylinder[i].gasmix.o2.permille == o2 && state->cur_dive->cylinder[i].gasmix.he.permille == he) {
+	for (i = 0; i < state->cur_dive->cylinders.nr; ++i) {
+		const cylinder_t *cyl = get_cylinder(state->cur_dive, i);
+		if (cyl->gasmix.o2.permille == o2 && cyl->gasmix.he.permille == he) {
 			found = true;
 			break;
 		}
 	}
 	if (!found) {
 		// Cylinder not found, creating a new one
-		cylinder_start(state);
-		state->cur_dive->cylinder[state->cur_cylinder_index].gasmix.o2.permille = o2;
-		state->cur_dive->cylinder[state->cur_cylinder_index].gasmix.he.permille = he;
+		cyl = cylinder_start(state);
+		cyl->gasmix.o2.permille = o2;
+		cyl->gasmix.he.permille = he;
 		cylinder_end(state);
-		i = state->cur_cylinder_index;
 	}
 
-	add_gas_switch_event(state->cur_dive, get_dc(state), state->sample_rate ? atoi(data[0]) / state->sample_rate * 10 : atoi(data[0]), i);
+	add_gas_switch_event(state->cur_dive, get_dc(state), state->sample_rate ? atoi(data[0]) / state->sample_rate * 10 : atoi(data[0]), state->cur_dive->cylinders.nr - 1);
 	return 0;
 }
 
@@ -201,7 +203,6 @@ static int shearwater_dive(void *param, int columns, char **data, char **column)
 	int retval = 0;
 	struct parser_state *state = (struct parser_state *)param;
 	sqlite3 *handle = state->sql_handle;
-	char *err = NULL;
 	char get_profile_template[] = "select currentTime,currentDepth,waterTemp,averagePPO2,currentNdl,CNSPercent,decoCeiling,firstStopDepth,firstStopTime from dive_log_records where diveLogId=%ld";
 	char get_profile_template_ai[] = "select currentTime,currentDepth,waterTemp,averagePPO2,currentNdl,CNSPercent,decoCeiling,aiSensor0_PressurePSI,aiSensor1_PressurePSI,firstStopDepth,firstStopTime from dive_log_records where diveLogId = %ld";
 	char get_cylinder_template[] = "select fractionO2,fractionHe from dive_log_records where diveLogId = %ld group by fractionO2,fractionHe";
@@ -276,7 +277,7 @@ static int shearwater_dive(void *param, int columns, char **data, char **column)
 
 	if (data[11]) {
 		snprintf(get_buffer, sizeof(get_buffer) - 1, get_mode_template, dive_id);
-		retval = sqlite3_exec(handle, get_buffer, &shearwater_mode, state, &err);
+		retval = sqlite3_exec(handle, get_buffer, &shearwater_mode, state, NULL);
 		if (retval != SQLITE_OK) {
 			fprintf(stderr, "%s", "Database query shearwater_mode failed.\n");
 			return 1;
@@ -284,24 +285,24 @@ static int shearwater_dive(void *param, int columns, char **data, char **column)
 	}
 
 	snprintf(get_buffer, sizeof(get_buffer) - 1, get_cylinder_template, dive_id);
-	retval = sqlite3_exec(handle, get_buffer, &shearwater_cylinders, state, &err);
+	retval = sqlite3_exec(handle, get_buffer, &shearwater_cylinders, state, NULL);
 	if (retval != SQLITE_OK) {
 		fprintf(stderr, "%s", "Database query shearwater_cylinders failed.\n");
 		return 1;
 	}
 
 	snprintf(get_buffer, sizeof(get_buffer) - 1, get_changes_template, dive_id);
-	retval = sqlite3_exec(handle, get_buffer, &shearwater_changes, state, &err);
+	retval = sqlite3_exec(handle, get_buffer, &shearwater_changes, state, NULL);
 	if (retval != SQLITE_OK) {
 		fprintf(stderr, "%s", "Database query shearwater_changes failed.\n");
 		return 1;
 	}
 
 	snprintf(get_buffer, sizeof(get_buffer) - 1, get_profile_template_ai, dive_id);
-	retval = sqlite3_exec(handle, get_buffer, &shearwater_ai_profile_sample, state, &err);
+	retval = sqlite3_exec(handle, get_buffer, &shearwater_ai_profile_sample, state, NULL);
 	if (retval != SQLITE_OK) {
 		snprintf(get_buffer, sizeof(get_buffer) - 1, get_profile_template, dive_id);
-		retval = sqlite3_exec(handle, get_buffer, &shearwater_profile_sample, state, &err);
+		retval = sqlite3_exec(handle, get_buffer, &shearwater_profile_sample, state, NULL);
 		if (retval != SQLITE_OK) {
 			fprintf(stderr, "%s", "Database query shearwater_profile_sample failed.\n");
 			return 1;
@@ -321,7 +322,6 @@ static int shearwater_cloud_dive(void *param, int columns, char **data, char **c
 	int retval = 0;
 	struct parser_state *state = (struct parser_state *)param;
 	sqlite3 *handle = state->sql_handle;
-	char *err = NULL;
 	char get_profile_template[] = "select currentTime,currentDepth,waterTemp,averagePPO2,currentNdl,CNSPercent,decoCeiling,firstStopDepth,firstStopTime from dive_log_records where diveLogId=%ld";
 	char get_profile_template_ai[] = "select currentTime,currentDepth,waterTemp,averagePPO2,currentNdl,CNSPercent,decoCeiling,aiSensor0_PressurePSI,aiSensor1_PressurePSI,firstStopDepth,firstStopTime from dive_log_records where diveLogId = %ld";
 	char get_cylinder_template[] = "select fractionO2 / 100,fractionHe / 100 from dive_log_records where diveLogId = %ld group by fractionO2,fractionHe";
@@ -400,7 +400,7 @@ static int shearwater_cloud_dive(void *param, int columns, char **data, char **c
 
 	if (data[11]) {
 		snprintf(get_buffer, sizeof(get_buffer) - 1, get_mode_template, dive_id);
-		retval = sqlite3_exec(handle, get_buffer, &shearwater_mode, state, &err);
+		retval = sqlite3_exec(handle, get_buffer, &shearwater_mode, state, NULL);
 		if (retval != SQLITE_OK) {
 			fprintf(stderr, "%s", "Database query shearwater_mode failed.\n");
 			return 1;
@@ -408,24 +408,24 @@ static int shearwater_cloud_dive(void *param, int columns, char **data, char **c
 	}
 
 	snprintf(get_buffer, sizeof(get_buffer) - 1, get_cylinder_template, dive_id);
-	retval = sqlite3_exec(handle, get_buffer, &shearwater_cylinders, state, &err);
+	retval = sqlite3_exec(handle, get_buffer, &shearwater_cylinders, state, NULL);
 	if (retval != SQLITE_OK) {
 		fprintf(stderr, "%s", "Database query shearwater_cylinders failed.\n");
 		return 1;
 	}
 
 	snprintf(get_buffer, sizeof(get_buffer) - 1, get_changes_template, dive_id);
-	retval = sqlite3_exec(handle, get_buffer, &shearwater_changes, state, &err);
+	retval = sqlite3_exec(handle, get_buffer, &shearwater_changes, state, NULL);
 	if (retval != SQLITE_OK) {
 		fprintf(stderr, "%s", "Database query shearwater_changes failed.\n");
 		return 1;
 	}
 
 	snprintf(get_buffer, sizeof(get_buffer) - 1, get_profile_template_ai, dive_id);
-	retval = sqlite3_exec(handle, get_buffer, &shearwater_ai_profile_sample, state, &err);
+	retval = sqlite3_exec(handle, get_buffer, &shearwater_ai_profile_sample, state, NULL);
 	if (retval != SQLITE_OK) {
 		snprintf(get_buffer, sizeof(get_buffer) - 1, get_profile_template, dive_id);
-		retval = sqlite3_exec(handle, get_buffer, &shearwater_profile_sample, state, &err);
+		retval = sqlite3_exec(handle, get_buffer, &shearwater_profile_sample, state, NULL);
 		if (retval != SQLITE_OK) {
 			fprintf(stderr, "%s", "Database query shearwater_profile_sample failed.\n");
 			return 1;
@@ -444,7 +444,6 @@ int parse_shearwater_buffer(sqlite3 *handle, const char *url, const char *buffer
 	UNUSED(size);
 
 	int retval;
-	char *err = NULL;
 	struct parser_state state;
 
 	init_parser_state(&state);
@@ -458,7 +457,7 @@ int parse_shearwater_buffer(sqlite3 *handle, const char *url, const char *buffer
 
 	char get_dives[] = "select l.number,timestamp,location||' / '||site,buddy,notes,imperialUnits,maxDepth,maxTime,startSurfacePressure,computerSerial,computerModel,i.diveId FROM dive_info AS i JOIN dive_logs AS l ON i.diveId=l.diveId";
 
-	retval = sqlite3_exec(handle, get_dives, &shearwater_dive, &state, &err);
+	retval = sqlite3_exec(handle, get_dives, &shearwater_dive, &state, NULL);
 	free_parser_state(&state);
 
 	if (retval != SQLITE_OK) {
@@ -476,7 +475,6 @@ int parse_shearwater_cloud_buffer(sqlite3 *handle, const char *url, const char *
 	UNUSED(size);
 
 	int retval;
-	char *err = NULL;
 	struct parser_state state;
 
 	init_parser_state(&state);
@@ -487,7 +485,7 @@ int parse_shearwater_cloud_buffer(sqlite3 *handle, const char *url, const char *
 
 	char get_dives[] = "select l.number,strftime('%s', DiveDate),location||' / '||site,buddy,notes,imperialUnits,maxDepth,maxTime,startSurfacePressure,computerSerial,computerModel,d.diveId,l.sampleRateMs FROM dive_details AS d JOIN dive_logs AS l ON d.diveId=l.diveId";
 
-	retval = sqlite3_exec(handle, get_dives, &shearwater_cloud_dive, &state, &err);
+	retval = sqlite3_exec(handle, get_dives, &shearwater_cloud_dive, &state, NULL);
 	free_parser_state(&state);
 
 	if (retval != SQLITE_OK) {
