@@ -3,6 +3,7 @@
 #include "dive.h"
 #include "core/settings/qPrefLanguage.h"
 #include "core/settings/qPrefUpdateManager.h"
+#include "core/subsurface-qt/divelistnotifier.h"
 #include "subsurface-string.h"
 #include "subsurface-string.h"
 #include "gettextfromc.h"
@@ -486,26 +487,26 @@ void initUiLanguage()
 	free((void*)prefs.locale.lang_locale);
 	prefs.locale.lang_locale = copy_qstring(uiLang);
 
-	if (!prefs.date_format_override || empty_string(prefs.date_format_short) || empty_string(prefs.date_format)) {
+	if (!prefs.date_format_override || empty_string(prefs.date_format)) {
 		// derive our standard date format from what the locale gives us
-		// the short format is fine
 		// the long format uses long weekday and month names, so replace those with the short ones
 		// for time we don't want the time zone designator and don't want leading zeroes on the hours
-		shortDateFormat = loc.dateFormat(QLocale::ShortFormat);
 		dateFormat = loc.dateFormat(QLocale::LongFormat);
 		dateFormat.replace("dddd,", "ddd").replace("dddd", "ddd").replace("MMMM", "MMM");
 		// special hack for Swedish as our switching from long weekday names to short weekday names
 		// messes things up there
 		dateFormat.replace("'en' 'den' d:'e'", " d");
-		if (!prefs.date_format_override || empty_string(prefs.date_format)) {
-			free((void *)prefs.date_format);
-			prefs.date_format = copy_qstring(dateFormat);
-		}
-		if (!prefs.date_format_override || empty_string(prefs.date_format_short)) {
-			free((void *)prefs.date_format_short);
-			prefs.date_format_short = copy_qstring(shortDateFormat);
-		}
+		free((void *)prefs.date_format);
+		prefs.date_format = copy_qstring(dateFormat);
 	}
+
+	if (!prefs.date_format_override || empty_string(prefs.date_format_short)) {
+		// derive our standard date format from what the locale gives us
+		shortDateFormat = loc.dateFormat(QLocale::ShortFormat);
+		free((void *)prefs.date_format_short);
+		prefs.date_format_short = copy_qstring(shortDateFormat);
+	}
+
 	if (!prefs.time_format_override || empty_string(prefs.time_format)) {
 		timeFormat = loc.timeFormat();
 		timeFormat.replace("(t)", "").replace(" t", "").replace("t", "").replace("hh", "h").replace("HH", "H").replace("'kl'.", "");
@@ -621,6 +622,23 @@ QString get_pressure_string(pressure_t pressure, bool showunit)
 	}
 }
 
+QString get_salinity_string(int salinity)
+{
+        return QStringLiteral("%L1%2").arg(salinity / 10.0).arg(gettextFromC::tr("g/ℓ"));
+}
+
+QString get_water_type_string(int salinity)
+{
+	if (salinity < 10050)
+		return waterTypes[FRESHWATER];
+	else if (salinity < 10190)
+		return waterTypes[BRACKISHWATER];
+	else if (salinity < 10210)
+		return waterTypes[EN13319WATER];
+	else
+		return waterTypes[SALTWATER];
+}
+
 QString getSubsurfaceDataPath(QString folderToFind)
 {
 	QString execdir;
@@ -678,12 +696,24 @@ int gettimezoneoffset(timestamp_t when)
 	if (when == 0)
 		dt1 = QDateTime::currentDateTime();
 	else
-		dt1 = QDateTime::fromMSecsSinceEpoch(when * 1000);
+		dt1 = timestampToDateTime(when);
 	dt2 = dt1.toUTC();
 	dt1.setTimeSpec(Qt::UTC);
 	return dt2.secsTo(dt1);
 }
 
+QDateTime timestampToDateTime(timestamp_t when)
+{
+	// Subsurface always uses "local time" as in "whatever was the local time at the location"
+	// so all time stamps have no time zone information and are in UTC
+	return QDateTime::fromMSecsSinceEpoch(1000 * when, Qt::UTC);
+}
+
+timestamp_t dateTimeToTimestamp(const QDateTime &t)
+{
+	return t.toSecsSinceEpoch();
+}
+
 QString render_seconds_to_string(int seconds)
 {
 	if (seconds % 60 == 0)
@@ -964,10 +994,7 @@ extern "C" char *get_current_date()
 
 QString get_trip_date_string(timestamp_t when, int nr, bool getday)
 {
-	struct tm tm;
-	utc_mkdate(when, &tm);
-	QDateTime localTime = QDateTime::fromMSecsSinceEpoch(1000*when,Qt::UTC);
-	localTime.setTimeSpec(Qt::UTC);
+	QDateTime localTime = timestampToDateTime(when);
 
 	QString suffix = " " + gettextFromC::tr("(%n dive(s))", "", nr);
 	if (getday)
@@ -1143,6 +1170,11 @@ QString localFilePath(const QString &originalFilename)
 	return localFilenameOf.value(originalFilename, originalFilename);
 }
 
+// the water types need to match the watertypes enum
+const QStringList waterTypes = {
+	gettextFromC::tr("Fresh"), gettextFromC::tr("Brackish"), gettextFromC::tr("EN13319"), gettextFromC::tr("Salt"), gettextFromC::tr("Use DC")
+};
+
 // TODO: Apparently Qt has no simple way of listing the supported video
 // codecs? Do we have to query them by hand using QMediaPlayer::hasSupport()?
 const QStringList videoExtensionsList = {
@@ -1579,31 +1611,6 @@ char *intdup(int index)
 	return strdup(tmpbuf);
 }
 
-QHash<int, double> factor_cache;
-
-QReadWriteLock factorCacheLock;
-extern "C" double cache_value(int tissue, int timestep, enum inertgas inertgas)
-{
-	double value;
-	int key = (timestep << 5) + (tissue << 1);
-	if (inertgas == HE)
-		++key;
-	factorCacheLock.lockForRead();
-	value = factor_cache.value(key);
-	factorCacheLock.unlock();
-	return value;
-}
-
-extern "C" void cache_insert(int tissue, int timestep, enum inertgas inertgas, double value)
-{
-	int key = (timestep << 5) + (tissue << 1);
-	if (inertgas == HE)
-		++key;
-	factorCacheLock.lockForWrite();
-	factor_cache.insert(key, value);
-	factorCacheLock.unlock();
-}
-
 extern "C" void print_qt_versions()
 {
 	printf("%s\n", qPrintable(QStringLiteral("built with Qt Version %1, runtime from Qt Version %2").arg(QT_VERSION_STR).arg(qVersion())));
@@ -1678,3 +1685,8 @@ std::vector<int> get_cylinder_map_for_add(int count, int n)
 	std::iota(mapping.begin() + n, mapping.end(), n + 1);
 	return mapping;
 }
+
+extern "C" void emit_reset_signal()
+{
+	emit diveListNotifier.dataReset();
+}
