@@ -6,6 +6,7 @@
 #define DIVELISTNOTIFIER_H
 
 #include "core/dive.h"
+#include "core/pictureobj.h"
 
 #include <QObject>
 
@@ -77,6 +78,9 @@ struct TripField {
 class DiveListNotifier : public QObject {
 	Q_OBJECT
 signals:
+	// The core structures were completely reset. Repopulate all models.
+	void dataReset();
+
 	// Note that there are no signals for trips being added and created
 	// because these events never happen without a dive being added, removed or moved.
 	// The dives are always sorted according to the dives_less_than() function of the core.
@@ -116,6 +120,11 @@ signals:
 	// Event-related signals. Currently, we're very blunt: only one signal for any changes to the events.
 	void eventsChanged(dive *d);
 
+	// Picture (media) related signals
+	void pictureOffsetChanged(dive *d, QString filename, offset_t offset);
+	void picturesRemoved(dive *d, QVector<QString> filenames);
+	void picturesAdded(dive *d, QVector<PictureObj> pics);
+
 	// This signal is emited every time a command is executed.
 	// This is used to hide an old multi-dives-edited warning message.
 	// This is necessary, so that the user can't click on the "undo" button and undo
