@@ -73,6 +73,7 @@ struct preferences default_prefs = {
 	.display_duration = true,
 	.display_transitions = true,
 	.display_variations = false,
+	.o2narcotic = true,
 	.safetystop = true,
 	.bottomsac = 20000,
 	.decosac = 17000,
@@ -100,11 +101,12 @@ struct preferences default_prefs = {
 	.auto_recalculate_thumbnails = true,
 	.extract_video_thumbnails = true,
 	.extract_video_thumbnails_position = 20,		// The first fifth seems like a reasonable place
-	.filterCaseSensitive = false,
-	.filterFullTextNotes = true,
 };
 
-int run_survey;
+int ignore_bt;
+#ifdef SUBSURFACE_MOBILE_DESKTOP
+char *testqml = NULL;
+#endif
 
 const struct units *get_units()
 {
@@ -126,9 +128,9 @@ const char *monthname(int mon)
  */
 bool imported = false;
 
-bool version_printed = false;
 void print_version()
 {
+	static bool version_printed = false;
 	if (version_printed)
 		return;
 	printf("Subsurface v%s,\n", subsurface_git_version());
@@ -177,11 +179,14 @@ static void print_help()
 	printf("\nUsage: subsurface [options] [logfile ...] [--import logfile ...]");
 	printf("\n\noptions include:");
 	printf("\n --help|-h             This help text");
+	printf("\n --ignore-bt           Don't enable Bluetooth support");
 	printf("\n --import logfile ...  Logs before this option is treated as base, everything after is imported");
 	printf("\n --verbose|-v          Verbose debug (repeat to increase verbosity)");
 	printf("\n --version             Prints current version");
-	printf("\n --survey              Offer to submit a user survey");
 	printf("\n --user=<test>         Choose configuration space for user <test>");
+#ifdef SUBSURFACE_MOBILE_DESKTOP
+	printf("\n --testqml=<dir>       Use QML files from <dir> instead of QML resources");
+#endif
 	printf("\n --cloud-timeout=<nr>  Set timeout for cloud connection (0 < timeout < 60)\n\n");
 }
 
@@ -219,6 +224,10 @@ void parse_argument(const char *arg)
 				print_help();
 				exit(0);
 			}
+			if (strcmp(arg, "--ignore-bt") == 0) {
+				ignore_bt = true;
+				return;
+			}
 			if (strcmp(arg, "--import") == 0) {
 				imported = true; /* mark the dives so far as the base, * everything after is imported */
 				return;
@@ -232,14 +241,17 @@ void parse_argument(const char *arg)
 				print_version();
 				exit(0);
 			}
-			if (strcmp(arg, "--survey") == 0) {
-				run_survey = true;
-				return;
-			}
 			if (strcmp(arg, "--allow_run_as_root") == 0) {
 				++force_root;
 				return;
 			}
+#ifdef SUBSURFACE_MOBILE_DESKTOP
+			if (strncmp(arg, "--testqml=", sizeof("--testqml=") - 1) == 0) {
+				testqml = malloc(strlen(arg) - sizeof("--testqml=") + 1);
+				strcpy(testqml, arg + sizeof("--testqml=") - 1);
+				return;
+			}
+#endif
 		/* fallthrough */
 		case 'p':
 			/* ignore process serial number argument when run as native macosx app */
