@@ -3,7 +3,6 @@
 #define LOCATIONINFORMATION_H
 
 #include "core/units.h"
-#include "core/divesite.h"
 #include "ui_locationinformation.h"
 #include "modeldelegates.h"
 #include "qt-models/divelocationmodel.h"
@@ -11,12 +10,15 @@
 #include <QAbstractListModel>
 #include <QSortFilterProxyModel>
 
+class dive_site;
+
 class LocationInformationWidget : public QGroupBox {
 	Q_OBJECT
 public:
 	LocationInformationWidget(QWidget *parent = 0);
 	bool eventFilter(QObject*, QEvent*) override;
 	void initFields(dive_site *ds);
+	Ui::LocationInformation ui;
 
 protected:
 	void enableLocationButtons(bool enable);
@@ -25,12 +27,14 @@ public slots:
 	void acceptChanges();
 	void on_diveSiteCountry_editingFinished();
 	void on_diveSiteCoordinates_editingFinished();
+	void on_diveSiteCoordinates_textEdited(const QString &s);
 	void on_diveSiteDescription_editingFinished();
 	void on_diveSiteName_editingFinished();
 	void on_diveSiteNotes_editingFinished();
 	void on_diveSiteDistance_textChanged(const QString &s);
 	void reverseGeocode();
 	void mergeSelectedDiveSites();
+	void on_GPSbutton_clicked();
 private slots:
 	void updateLabels();
 	void diveSiteChanged(struct dive_site *ds, int field);
@@ -39,7 +43,7 @@ private slots:
 private:
 	void keyPressEvent(QKeyEvent *e) override;
 	void clearLabels();
-	Ui::LocationInformation ui;
+	void coordinatesSetWarning(bool warn);
 	GPSLocationInformationModel filter_model;
 	dive_site *diveSite;
 	int64_t closeDistance; // Distance of "close" dive sites in mm
