@@ -206,7 +206,6 @@ MainWindow::MainWindow() : QMainWindow(),
 	ui.menuFile->insertSeparator(ui.actionQuit);
 	connect(DivePlannerPointsModel::instance(), SIGNAL(planCreated()), this, SLOT(planCreated()));
 	connect(DivePlannerPointsModel::instance(), SIGNAL(planCanceled()), this, SLOT(planCanceled()));
-	connect(DivePlannerPointsModel::instance(), SIGNAL(variationsComputed(QString)), this, SLOT(updateVariations(QString)));
 	connect(plannerDetails->printPlan(), SIGNAL(pressed()), divePlannerWidget, SLOT(printDecoPlan()));
 	connect(this, &MainWindow::showError, ui.mainErrorMessage, &NotificationWidget::showError, Qt::AutoConnection);
 
@@ -406,18 +405,13 @@ MainWindow *MainWindow::instance()
 }
 
 // This gets called after one or more dives were added, edited or downloaded for a dive computer
-void MainWindow::refreshDisplay(bool doRecreateDiveList)
+void MainWindow::refreshDisplay()
 {
 	mainTab->reload();
-	TankInfoModel::instance()->update();
-	if (doRecreateDiveList)
-		diveList->reload();
 
-	MapWidget::instance()->reload();
 	setApplicationState(ApplicationState::Default);
 	diveList->setEnabled(true);
 	diveList->setFocus();
-	WSInfoModel::instance()->update();
 	ui.actionAutoGroup->setChecked(autogroup);
 }
 
@@ -450,16 +444,12 @@ void MainWindow::configureToolbar()
 
 void MainWindow::selectionChanged()
 {
-	if (!current_dive) {
-		mainTab->clearTabs();
-		mainTab->updateDiveInfo();
-		graphics->setEmptyState();
-	} else {
-		graphics->plotDive(nullptr, false, true);
-		mainTab->updateDiveInfo();
+	mainTab->updateDiveInfo();
+	if (current_dive) {
 		configureToolbar();
 		enableDisableOtherDCsActions();
 	}
+	graphics->plotDive(current_dive, false);
 	MapWidget::instance()->selectionChanged();
 }
 
@@ -652,7 +642,7 @@ void MainWindow::closeCurrentFile()
 {
 	/* free the dives and trips */
 	clear_git_id();
-	MultiFilterSortModel::instance()->clear(); // this clears all the core data structures
+	clear_dive_file_data(); // this clears all the core data structures and resets the models
 	setCurrentFile(nullptr);
 	diveList->setSortOrder(DiveTripModelBase::NR, Qt::DescendingOrder);
 	MapWidget::instance()->reload();
@@ -681,7 +671,6 @@ void MainWindow::on_actionClose_triggered()
 {
 	if (okToClose(tr("Please save or cancel the current dive edit before closing the file."))) {
 		closeCurrentFile();
-		DivePictureModel::instance()->updateDivePictures();
 		setApplicationState(ApplicationState::Default);
 	}
 }
@@ -741,21 +730,9 @@ void MainWindow::on_actionPreferences_triggered()
 
 void MainWindow::on_actionQuit_triggered()
 {
-	if (mainTab->isEditing()) {
-		mainTab->rejectChanges();
-		if (mainTab->isEditing())
-			// didn't discard the edits
-			return;
-	}
-	if (DivePlannerPointsModel::instance()->currentMode() != DivePlannerPointsModel::NOTHING) {
-		DivePlannerPointsModel::instance()->cancelPlan();
-		if (DivePlannerPointsModel::instance()->currentMode() != DivePlannerPointsModel::NOTHING)
-			// The planned dive was not discarded
-			return;
-	}
-
-	if (unsavedChanges() && (askSaveChanges() == false))
+	if (!okToClose(tr("Please save or cancel the current dive edit before quiting the application.")))
 		return;
+
 	writeSettings();
 	QApplication::quit();
 }
@@ -795,8 +772,7 @@ void MainWindow::refreshProfile()
 {
 	showProfile();
 	configureToolbar();
-	graphics->replot(current_dive);
-	DivePictureModel::instance()->updateDivePictures();
+	graphics->plotDive(current_dive, true);
 }
 
 void MainWindow::planCanceled()
@@ -804,7 +780,7 @@ void MainWindow::planCanceled()
 	// while planning we might have modified the displayed_dive
 	// let's refresh what's shown on the profile
 	refreshProfile();
-	refreshDisplay(false);
+	refreshDisplay();
 }
 
 void MainWindow::planCreated()
@@ -816,25 +792,20 @@ void MainWindow::planCreated()
 	diveList->setFocus();
 }
 
-void MainWindow::setPlanNotes()
+void MainWindow::setPlanNotes(QString plan)
 {
-	plannerDetails->divePlanOutput()->setHtml(displayed_dive.notes);
-}
-
-void MainWindow::updateVariations(QString variations)
-{
-	QString notes = QString(displayed_dive.notes);
-	free(displayed_dive.notes);
-	displayed_dive.notes = copy_qstring(notes.replace("VARIATIONS", variations));
-	plannerDetails->divePlanOutput()->setHtml(displayed_dive.notes);
+	plannerDetails->divePlanOutput()->setHtml(plan);
 }
 
 void MainWindow::printPlan()
 {
 #ifndef NO_PRINTING
 	char *disclaimer = get_planner_disclaimer_formatted();
+	// Prepend a logo and a disclaimer to the plan.
+	// Save the old plan so that it can be restored at the end of the function.
+	QString origPlan = plannerDetails->divePlanOutput()->toHtml();
 	QString diveplan = QStringLiteral("<img height=50 src=\":subsurface-icon\"> ") +
-			   QString(disclaimer) + plannerDetails->divePlanOutput()->toHtml();
+			   QString(disclaimer) + origPlan;
 	free(disclaimer);
 
 	QPrinter printer;
@@ -873,23 +844,10 @@ void MainWindow::printPlan()
 
 	plannerDetails->divePlanOutput()->setHtml(diveplan);
 	plannerDetails->divePlanOutput()->print(&printer);
-	plannerDetails->divePlanOutput()->setHtml(displayed_dive.notes);
+	plannerDetails->divePlanOutput()->setHtml(origPlan); // restore original plan
 #endif
 }
 
-void MainWindow::setupForAddAndPlan(const char *model)
-{
-	// clean out the dive and give it an id and the correct dc model
-	clear_dive(&displayed_dive);
-	displayed_dive.id = dive_getUniqID();
-	displayed_dive.when = QDateTime::currentMSecsSinceEpoch() / 1000L + gettimezoneoffset() + 3600;
-	displayed_dive.dc.model = strdup(model); // don't translate! this is stored in the XML file
-	dc_number = 1;
-	// setup the dive cylinders
-	DivePlannerPointsModel::instance()->clear();
-	DivePlannerPointsModel::instance()->setupCylinders();
-}
-
 void MainWindow::on_actionReplanDive_triggered()
 {
 	if (!plannerStateClean() || !current_dive)
@@ -907,7 +865,7 @@ void MainWindow::on_actionReplanDive_triggered()
 	graphics->clearHandlers();
 	setApplicationState(ApplicationState::PlanDive);
 	divePlannerWidget->setReplanButton(true);
-	divePlannerWidget->setupStartTime(QDateTime::fromMSecsSinceEpoch(1000 * current_dive->when, Qt::UTC));
+	divePlannerWidget->setupStartTime(timestampToDateTime(current_dive->when));
 	if (current_dive->surface_pressure.mbar)
 		divePlannerWidget->setSurfacePressure(current_dive->surface_pressure.mbar);
 	if (current_dive->salinity)
@@ -927,10 +885,9 @@ void MainWindow::on_actionDivePlanner_triggered()
 	setApplicationState(ApplicationState::PlanDive);
 
 	graphics->setPlanState();
+	dc_number = 1;
 
 	// create a simple starting dive, using the first gas from the just copied cylinders
-	setupForAddAndPlan("planned dive"); // don't translate, stored in XML file
-	DivePlannerPointsModel::instance()->setupStartTime();
 	DivePlannerPointsModel::instance()->createSimpleDive();
 	// plan the dive in the same mode as the currently selected one
 	if (current_dive) {
@@ -939,7 +896,6 @@ void MainWindow::on_actionDivePlanner_triggered()
 		if (current_dive->salinity)
 			divePlannerWidget->setSalinity(current_dive->salinity);
 	}
-	DivePictureModel::instance()->updateDivePictures();
 	divePlannerWidget->setReplanButton(false);
 }
 
@@ -965,8 +921,8 @@ void MainWindow::on_actionAddDive_triggered()
 
 void MainWindow::on_actionRenumber_triggered()
 {
-	RenumberDialog::instance()->renumberOnlySelected(false);
-	RenumberDialog::instance()->show();
+	RenumberDialog dialog(false, this);
+	dialog.exec();
 }
 
 void MainWindow::on_actionAutoGroup_triggered()
@@ -1124,7 +1080,7 @@ void MainWindow::on_actionPreviousDC_triggered()
 	unsigned nrdc = number_of_computers(current_dive);
 	dc_number = (dc_number + nrdc - 1) % nrdc;
 	configureToolbar();
-	graphics->plotDive(nullptr, false, true);
+	graphics->plotDive(current_dive, false);
 	mainTab->updateDiveInfo();
 }
 
@@ -1133,7 +1089,7 @@ void MainWindow::on_actionNextDC_triggered()
 	unsigned nrdc = number_of_computers(current_dive);
 	dc_number = (dc_number + 1) % nrdc;
 	configureToolbar();
-	graphics->plotDive(nullptr, false, true);
+	graphics->plotDive(current_dive, false);
 	mainTab->updateDiveInfo();
 }
 
@@ -1707,31 +1663,21 @@ void MainWindow::on_actionImportDiveSites_triggered()
 
 void MainWindow::editCurrentDive()
 {
-	if (!current_dive)
+	// We only allow editing of the profile for manually added dives.
+	if (!current_dive || !same_string(current_dive->dc.model, "manually added dive"))
 		return;
 
-	if (mainTab->isEditing() || DivePlannerPointsModel::instance()->currentMode() != DivePlannerPointsModel::NOTHING) {
-		QMessageBox::warning(this, tr("Warning"), tr("Please, first finish the current edition before trying to do another."));
+	// This shouldn't be possible, but let's make sure no weird "double editing" takes place.
+	if (mainTab->isEditing() || DivePlannerPointsModel::instance()->currentMode() != DivePlannerPointsModel::NOTHING)
 		return;
-	}
 
-	struct dive *d = current_dive;
-	QString defaultDC(d->dc.model);
+	disableShortcuts();
 	DivePlannerPointsModel::instance()->clear();
-	if (defaultDC == "manually added dive") {
-		disableShortcuts();
-		DivePlannerPointsModel::instance()->setPlanMode(DivePlannerPointsModel::ADD);
-		graphics->setAddState();
-		setApplicationState(ApplicationState::EditDive);
-		DivePlannerPointsModel::instance()->loadFromDive(d);
-		mainTab->enableEdition();
-	} else if (defaultDC == "planned dive") {
-		disableShortcuts();
-		DivePlannerPointsModel::instance()->setPlanMode(DivePlannerPointsModel::PLAN);
-		setApplicationState(ApplicationState::EditPlannedDive);
-		DivePlannerPointsModel::instance()->loadFromDive(d);
-		mainTab->enableEdition();
-	}
+	DivePlannerPointsModel::instance()->setPlanMode(DivePlannerPointsModel::ADD);
+	graphics->setAddState();
+	setApplicationState(ApplicationState::EditDive);
+	DivePlannerPointsModel::instance()->loadFromDive(current_dive);
+	mainTab->enableEdition();
 }
 
 void MainWindow::turnOffNdlTts()
