@@ -12,7 +12,7 @@ namespace Ui {
 };
 
 class WeightModel;
-class CylindersModel;
+class CylindersModelFiltered;
 
 class TabDiveEquipment : public TabBase {
 	Q_OBJECT
@@ -21,9 +21,6 @@ public:
 	~TabDiveEquipment();
 	void updateData() override;
 	void clear() override;
-	void acceptChanges();
-	void rejectChanges();
-	void divesEdited(int i);
 	void closeWarning();
 
 private slots:
@@ -34,11 +31,12 @@ private slots:
 	void editCylinderWidget(const QModelIndex &index);
 	void editWeightWidget(const QModelIndex &index);
 	void on_suit_editingFinished();
+	void divesEdited(int count);
 
 private:
 	Ui::TabDiveEquipment ui;
 	SuitCompletionModel suitModel;
-	CylindersModel *cylindersModel;
+	CylindersModelFiltered *cylindersModel;
 	WeightModel *weightModel;
 };
 
