@@ -17,17 +17,16 @@
 #define TEXT_EDITED 1
 #define CSS_SET_HEADING_BLUE "QLabel { color: mediumblue;} "
 
-enum watertypes { FRESHWATER, SALTYWATER, EN13319WATER, SALTWATER, DC_WATERTYPE};
-
 TabDiveInformation::TabDiveInformation(QWidget *parent) : TabBase(parent), ui(new Ui::TabDiveInformation())
 {
 	ui->setupUi(this);
 	connect(&diveListNotifier, &DiveListNotifier::divesChanged, this, &TabDiveInformation::divesChanged);
-	QStringList atmPressTypes { "mbar", get_depth_unit() ,tr("use dc")};
+	connect(&diveListNotifier, &DiveListNotifier::cylinderAdded, this, &TabDiveInformation::cylinderChanged);
+	connect(&diveListNotifier, &DiveListNotifier::cylinderRemoved, this, &TabDiveInformation::cylinderChanged);
+	connect(&diveListNotifier, &DiveListNotifier::cylinderEdited, this, &TabDiveInformation::cylinderChanged);
+	QStringList atmPressTypes { "mbar", get_depth_unit() ,tr("Use DC")};
 	ui->atmPressType->insertItems(0, atmPressTypes);
 	pressTypeIndex = 0;
-	// the water types need to match the enum above
-	waterTypes = QStringList({tr("Fresh"), tr("Salty"), "EN13319", tr("Salt"), tr("use dc")});
 	ui->waterTypeCombo->insertItems(0, waterTypes);
 
 	// This needs to be the same order as enum dive_comp_type in dive.h!
@@ -120,8 +119,8 @@ void TabDiveInformation::updateWaterTypeWidget()
 // Update fields that depend on the dive profile
 void TabDiveInformation::updateProfile()
 {
-	ui->maxcnsText->setText(QString("%1\%").arg(current_dive->maxcns));
-	ui->otuText->setText(QString("%1").arg(current_dive->otu));
+	ui->maxcnsText->setText(QString("%L1\%").arg(current_dive->maxcns));
+	ui->otuText->setText(QString("%L1").arg(current_dive->otu));
 	ui->maximumDepthText->setText(get_depth_string(current_dive->maxdepth, true));
 	ui->averageDepthText->setText(get_depth_string(current_dive->meandepth, true));
 
@@ -183,7 +182,7 @@ int TabDiveInformation::updateSalinityComboIndex(int salinity)
 	else if (salinity < 10050)
 		return FRESHWATER;
 	else if (salinity < 10190)
-		return SALTYWATER;
+		return BRACKISHWATER;
 	else if (salinity < 10210)
 		return EN13319WATER;
 	else
@@ -229,24 +228,14 @@ void TabDiveInformation::updateData()
 	ui->airtemp->setText(get_temperature_string(current_dive->airtemp, true));
 	ui->atmPressType->setItemText(1, get_depth_unit());  // Check for changes in depth unit (imperial/metric)
 	ui->atmPressType->setCurrentIndex(0);                // Set the atmospheric pressure combo box to mbar
-	if (current_dive->user_salinity)
-		salinity_value = current_dive->user_salinity;
-	else
-		salinity_value = current_dive->salinity;
+	salinity_value = get_dive_salinity(current_dive);
 	if (salinity_value) {			// Set water type indicator (EN13319 = 1.020 g/l)
 		if (prefs.salinityEditDefault) {   //If edit-salinity is enabled then set correct water type in combobox:
 			ui->waterTypeCombo->setCurrentIndex(updateSalinityComboIndex(salinity_value));
 		} else {         // If water salinity is not editable: show water type as a text label
-			if (salinity_value < 10050)
-				ui->waterTypeText->setText(waterTypes[FRESHWATER]);
-			else if (salinity_value < 10190)
-				ui->waterTypeText->setText(waterTypes[SALTYWATER]);
-			else if (salinity_value < 10210)
-				ui->waterTypeText->setText(waterTypes[EN13319WATER]);
-			else
-				ui->waterTypeText->setText(waterTypes[SALTWATER]);
+                        ui->waterTypeText->setText(get_water_type_string(salinity_value));
 		}
-		ui->salinityText->setText(QString("%1g/ℓ").arg(salinity_value / 10.0));
+		ui->salinityText->setText(get_salinity_string(salinity_value));
 	} else {
 		ui->waterTypeCombo->setCurrentIndex(-1);
 		ui->waterTypeText->setText(tr("unknown"));
@@ -267,7 +256,8 @@ void TabDiveInformation::updateData()
 }
 
 // From the index of the water type combo box, set the dive->salinity to an appropriate value
-void TabDiveInformation::on_waterTypeCombo_activated(int index) {
+void TabDiveInformation::on_waterTypeCombo_activated(int index)
+{
 	Q_UNUSED(index)
 	int combobox_salinity = 0;
 	int dc_salinity = current_dc->salinity;
@@ -275,8 +265,8 @@ void TabDiveInformation::on_waterTypeCombo_activated(int index) {
 	case FRESHWATER:
 		combobox_salinity = FRESHWATER_SALINITY;
 		break;
-	case SALTYWATER:
-		combobox_salinity = 10100;
+	case BRACKISHWATER:
+		combobox_salinity = BRACKISH_SALINITY;
 		break;
 	case EN13319WATER:
 		combobox_salinity = EN13319_SALINITY;
@@ -295,7 +285,7 @@ void TabDiveInformation::on_waterTypeCombo_activated(int index) {
 	}
 	// Save and display the new salinity value
 	if (combobox_salinity)
-		ui->salinityText->setText(QString("%1g/ℓ").arg(combobox_salinity / 10.0));
+		ui->salinityText->setText(get_salinity_string(combobox_salinity));
 	else
 		ui->salinityText->clear();
 	divesEdited(Command::editWaterTypeUser(combobox_salinity, false));
@@ -306,6 +296,14 @@ void TabDiveInformation::on_waterTypeCombo_activated(int index) {
 		ui->salinityOverWrittenIcon->setVisible(true);
 }
 
+void TabDiveInformation::cylinderChanged(dive *d)
+{
+	// If this isn't the current dive, do nothing
+	if (current_dive != d)
+		return;
+	updateProfile();
+}
+
 // This function gets called if a field gets updated by an undo command.
 // Refresh the corresponding UI field.
 void TabDiveInformation::divesChanged(const QVector<dive *> &dives, DiveField field)
@@ -346,11 +344,11 @@ void TabDiveInformation::divesChanged(const QVector<dive *> &dives, DiveField fi
 	else
 		salinity_value = current_dive->salinity;
 	ui->waterTypeCombo->setCurrentIndex(updateSalinityComboIndex(salinity_value));
-	ui->salinityText->setText(QString("%1g/ℓ").arg(salinity_value / 10.0));
+	ui->salinityText->setText(QString("%L1g/ℓ").arg(salinity_value / 10.0));
 	// TODO: The profile should recognize itself when the dive mode changed.
 	// It seem awkward to route this via the dive-information tab.
 	if (replot)
-		MainWindow::instance()->graphics->replot();
+		MainWindow::instance()->graphics->plotDive(current_dive, true);
 }
 
 void TabDiveInformation::on_visibility_valueChanged(int value)
@@ -412,12 +410,16 @@ void TabDiveInformation::on_watertemp_editingFinished()
 		divesEdited(Command::editWaterTemp(parseTemperatureToMkelvin(ui->watertemp->text()), false));
 }
 
-void TabDiveInformation::on_atmPressType_currentIndexChanged(int index) {
+void TabDiveInformation::on_atmPressType_currentIndexChanged(int index)
+{
 	Q_UNUSED(index)
 	updateTextBox(COMBO_CHANGED);
 }
 
-void TabDiveInformation::on_atmPressVal_editingFinished() { updateTextBox(TEXT_EDITED); }
+void TabDiveInformation::on_atmPressVal_editingFinished()
+{
+	updateTextBox(TEXT_EDITED);
+}
 
 void TabDiveInformation::updateTextBox(int event) // Either the text box has been edited or the pressure type has changed.
 {                                       // Either way this gets a numeric value and puts it on the text box atmPressVal,
@@ -466,5 +468,3 @@ void TabDiveInformation::updateTextBox(int event) // Either the text box has bee
 			divesEdited(Command::editAtmPress(atmpress.mbar, false));      // and save the pressure for undo
 	}
 }
-
-
