@@ -339,7 +339,7 @@ void QMLManager::applicationStateChanged(Qt::ApplicationState state)
 void QMLManager::openLocalThenRemote(QString url)
 {
 	// clear out the models and the fulltext index
-	MobileModels::instance()->clear();
+	clear_dive_file_data();
 	setDiveListProcessing(true);
 	setNotificationText(tr("Open local dive data file"));
 	appendTextToLog(QString("Open dive data file %1 - git_local only is %2").arg(url).arg(git_local_only));
@@ -383,7 +383,6 @@ void QMLManager::openLocalThenRemote(QString url)
 		// the following steps can take a long time, so provide updates
 		setNotificationText(tr("Processing %1 dives").arg(dive_table.nr));
 		process_loaded_dives();
-		MobileModels::instance()->reset();
 		setNotificationText(tr("%1 dives loaded from local dive data file").arg(dive_table.nr));
 	}
 	if (qPrefCloudStorage::cloud_verification_status() == qPrefCloudStorage::CS_NEED_TO_VERIFY) {
@@ -404,6 +403,8 @@ void QMLManager::openLocalThenRemote(QString url)
 
 	updateAllGlobalLists();
 	setDiveListProcessing(false);
+	// this could have added a new local cache directory
+	emit cloudCacheListChanged();
 }
 
 // Convenience function to accesss dive directly via its row.
@@ -535,6 +536,9 @@ void QMLManager::finishSetup()
 	}
 	m_initialized = true;
 	emit initializedChanged();
+	// this could have brought in new cache directories, so make sure QML
+	// calls our getter function again and doesn't show us outdated information
+	emit cloudCacheListChanged();
 }
 
 QMLManager::~QMLManager()
@@ -559,24 +563,27 @@ void QMLManager::saveCloudCredentials(const QString &newEmail, const QString &ne
 	bool cloudCredentialsChanged = false;
 	bool noCloud = qPrefCloudStorage::cloud_verification_status() == qPrefCloudStorage::CS_NOCLOUD;
 
+	// email address MUST be lower case or bad things happen
+	QString email = newEmail.toLower();
+
 	// make sure we only have letters, numbers, and +-_. in password and email address
 	QRegularExpression regExp("^[a-zA-Z0-9@.+_-]+$");
 	if (!noCloud) {
 		// in case of NO_CLOUD, the email address + passwd do not care, so do not check it.
 		if (newPassword.isEmpty() ||
 			!regExp.match(newPassword).hasMatch() ||
-			!regExp.match(newEmail).hasMatch()) {
+			!regExp.match(email).hasMatch()) {
 			setStartPageText(RED_FONT + tr("Cloud storage email and password can only consist of letters, numbers, and '.', '-', '_', and '+'.") + END_FONT);
 			return;
 		}
 		// use the same simplistic regex as the backend to check email addresses
 		regExp = QRegularExpression("^[a-zA-Z0-9.+_-]+@[a-zA-Z0-9.+_-]+\\.[a-zA-Z0-9]+");
-		if (!regExp.match(newEmail).hasMatch()) {
+		if (!regExp.match(email).hasMatch()) {
 			setStartPageText(RED_FONT + tr("Invalid format for email address") + END_FONT);
 			return;
 		}
 	}
-	if (!same_string(prefs.cloud_storage_email, qPrintable(newEmail))) {
+	if (!same_string(prefs.cloud_storage_email, qPrintable(email))) {
 		cloudCredentialsChanged = true;
 	}
 
@@ -590,11 +597,11 @@ void QMLManager::saveCloudCredentials(const QString &newEmail, const QString &ne
 		qPrefCloudStorage::set_cloud_verification_status(m_oldStatus);
 	}
 
-	if (!noCloud && !verifyCredentials(newEmail, newPassword, pin)) {
+	if (!noCloud && !verifyCredentials(email, newPassword, pin)) {
 		appendTextToLog("saveCloudCredentials: given cloud credentials didn't verify");
 		return;
 	}
-	qPrefCloudStorage::set_cloud_storage_email(newEmail);
+	qPrefCloudStorage::set_cloud_storage_email(email);
 	qPrefCloudStorage::set_cloud_storage_password(newPassword);
 
 	if (m_oldStatus == qPrefCloudStorage::CS_NOCLOUD && cloudCredentialsChanged && dive_table.nr) {
@@ -612,8 +619,7 @@ void QMLManager::saveCloudCredentials(const QString &newEmail, const QString &ne
 		syncLoadFromCloud();
 		manager()->clearAccessCache(); // remove any chached credentials
 		clear_git_id(); // invalidate our remembered GIT SHA
-		MobileModels::instance()->clear();
-		GpsListModel::instance()->clear();
+		clear_dive_file_data();
 		setStartPageText(tr("Attempting to open cloud storage with new credentials"));
 		// since we changed credentials, we need to try to connect to the cloud, regardless
 		// of whether we're in offline mode or not, to make sure the repository is synced
@@ -687,7 +693,7 @@ void QMLManager::loadDivesWithValidCredentials()
 		// if we aren't switching from no-cloud mode, let's clear the dive data
 		if (!noCloudToCloud) {
 			appendTextToLog("Clear out in memory dive data");
-			MobileModels::instance()->clear();
+			clear_dive_file_data();
 		} else {
 			appendTextToLog("Switching from no cloud mode; keep in memory dive data");
 		}
@@ -717,7 +723,6 @@ void QMLManager::loadDivesWithValidCredentials()
 	if (noCloudToCloud) {
 		git_storage_update_progress(qPrintable(tr("Loading dives from local storage ('no cloud' mode)")));
 		mergeLocalRepo();
-		MobileModels::instance()->reset();
 		appendTextToLog(QStringLiteral("%1 dives loaded after importing nocloud local storage").arg(dive_table.nr));
 		noCloudToCloud = false;
 		mark_divelist_changed(true);
@@ -779,7 +784,6 @@ void QMLManager::consumeFinishedLoad()
 	prefs.show_ccr_sensors = git_prefs.show_ccr_sensors;
 	prefs.pp_graphs.po2 = git_prefs.pp_graphs.po2;
 	process_loaded_dives();
-	MobileModels::instance()->reset();
 	appendTextToLog(QStringLiteral("%1 dives loaded").arg(dive_table.nr));
 	if (dive_table.nr == 0)
 		setStartPageText(tr("Cloud storage open successfully. No dives in dive list."));
@@ -787,7 +791,7 @@ void QMLManager::consumeFinishedLoad()
 
 void QMLManager::refreshDiveList()
 {
-	MobileModels::instance()->reset();
+	MobileModels::instance()->invalidate();
 }
 
 // Ouch. Editing a dive might create a dive site or change an existing dive site.
@@ -912,7 +916,7 @@ parsed:
 			// add a hundred years.
 			if (newDate.addYears(100) < QDateTime::currentDateTime().addYears(1))
 				newDate = newDate.addYears(100);
-			d->dc.when = d->when = newDate.toMSecsSinceEpoch() / 1000;
+			d->dc.when = d->when = dateTimeToTimestamp(newDate);
 			return true;
 		}
 		appendTextToLog("none of our parsing attempts worked for the date string");
@@ -2202,3 +2206,34 @@ void QMLManager::setDiveListProcessing(bool value)
 	}
 
 }
+
+void QMLManager::importCacheRepo(QString repo)
+{
+	struct dive_table table = empty_dive_table;
+	struct trip_table trips = empty_trip_table;
+	struct dive_site_table sites = empty_dive_site_table;
+	QString repoPath = QString("%1/cloudstorage/%2").arg(system_default_directory()).arg(repo);
+	appendTextToLog(QString("importing %1").arg(repoPath));
+	parse_file(qPrintable(repoPath), &table, &trips, &sites);
+	add_imported_dives(&table, &trips, &sites, IMPORT_MERGE_ALL_TRIPS);
+	changesNeedSaving();
+}
+
+QStringList QMLManager::cloudCacheList() const
+{
+	QDir localCacheDir(QString("%1/cloudstorage/").arg(system_default_directory()));
+	QStringList dirs = localCacheDir.entryList().filter(QRegExp("...+"));
+	QStringList result;
+	foreach(QString dir, dirs) {
+		QString originsDir = QString("%1/cloudstorage/%2/.git/refs/remotes/origin/").arg(system_default_directory()).arg(dir);
+		QDir remote(originsDir);
+		if (dir == "localrepo") {
+			result << QString("localrepo[master]");
+		} else {
+			foreach(QString branch, remote.entryList().filter(QRegExp("...+"))) {
+				result << QString("%1[%2]").arg(dir).arg(branch);
+			}
+		}
+	}
+	return result;
+}
