@@ -6,23 +6,44 @@
 #include "core/metrics.h"
 #include <QTransform>
 #include <QScreen>
+#include <QElapsedTimer>
+
+const double fontScale = 0.6; // profile looks less cluttered with smaller font
 
 QMLProfile::QMLProfile(QQuickItem *parent) :
 	QQuickPaintedItem(parent),
 	m_devicePixelRatio(1.0),
 	m_margin(0),
+	m_xOffset(0.0),
+	m_yOffset(0.0),
 	m_profileWidget(new ProfileWidget2)
 {
 	setAntialiasing(true);
+	setFlags(QQuickItem::ItemClipsChildrenToShape | QQuickItem::ItemHasContents );
 	m_profileWidget->setProfileState();
 	m_profileWidget->setPrintMode(true);
-	m_profileWidget->setFontPrintScale(0.8);
+	m_profileWidget->setFontPrintScale(fontScale);
 	connect(QMLManager::instance(), &QMLManager::sendScreenChanged, this, &QMLProfile::screenChanged);
+	connect(this, &QMLProfile::scaleChanged, this, &QMLProfile::triggerUpdate);
 	setDevicePixelRatio(QMLManager::instance()->lastDevicePixelRatio());
 }
 
+// we need this so we can connect update() to the scaleChanged() signal - which the connect above cannot do
+// directly as it chokes on the default parameter for update().
+// If the scale changes we may need to change our offsets to ensure that we still only show a subset of
+// the profile and not empty space around it, which the paint() method below will take care of, which will
+// eventually get called after we call update()
+void QMLProfile::triggerUpdate()
+{
+	update();
+}
+
 void QMLProfile::paint(QPainter *painter)
 {
+	QElapsedTimer timer;
+	if (verbose)
+		timer.start();
+
 	// let's look at the intended size of the content and scale our scene accordingly
 	QRect painterRect = painter->viewport();
 	QRect profileRect = m_profileWidget->viewport()->rect();
@@ -52,8 +73,27 @@ void QMLProfile::paint(QPainter *painter)
 	// shift the painter (taking its existing transformation into account)
 	QTransform profileTransform = QTransform();
 	profileTransform.scale(sx, sy);
+
+	// this is a bit complex because of the interaction with the Qt widget scaling
+	// logic. We use the default setup where the scale happens with the center of the
+	// widget as transformation center.
+	// The QML code *should* already ensure that we don't shrink the profile, but for
+	// the math below to work, let's make sure
+	qreal profileScale = scale();
+	if (profileScale < 1.0) {
+		profileScale = 1.0;
+		setScale(profileScale);
+	}
+	// When zooming and panning we want to ensure that we always show a subset of the
+	// profile and not the "empty space" around the profile.
+	// a bit of math on a piece of paper shows that our offsets need to stay within +/- dx and dy
+	qreal dx = width() * (profileScale - 1) / (2 * dpr * profileScale);
+	qreal dy = height() * (profileScale - 1) / (2 * dpr * profileScale);
+	m_xOffset = std::max(-dx, std::min(m_xOffset, dx));
+	m_yOffset = std::max(-dy, std::min(m_yOffset, dy));
+
 	QTransform painterTransform = painter->transform();
-	painterTransform.translate(-painterRect.width() * magicShiftFactor ,-painterRect.height() * magicShiftFactor);
+	painterTransform.translate(dpr * m_xOffset - painterRect.width() * magicShiftFactor, dpr * m_yOffset - painterRect.height() * magicShiftFactor);
 
 #if defined(PROFILE_SCALING_DEBUG)
 	// some debugging messages to help adjust this in case the magic above is insufficient
@@ -73,6 +113,8 @@ void QMLProfile::paint(QPainter *painter)
 
 	// finally, render the profile
 	m_profileWidget->render(painter);
+	if (verbose)
+		qDebug() << "finished rendering profile with offset" << QString::number(m_xOffset, 'f', 1) << "/" << QString::number(m_yOffset, 'f', 1)  << "in" << timer.elapsed() << "ms";
 }
 
 void QMLProfile::setMargin(int margin)
@@ -80,24 +122,29 @@ void QMLProfile::setMargin(int margin)
 	m_margin = margin;
 }
 
-QString QMLProfile::diveId() const
+int QMLProfile::diveId() const
 {
 	return m_diveId;
 }
 
-void QMLProfile::setDiveId(const QString &diveId)
+void QMLProfile::updateProfile()
 {
-	m_diveId = diveId;
-	struct dive *d = get_dive_by_uniq_id(m_diveId.toInt());
-	if (m_diveId.toInt() < 1)
-		return;
+	struct dive *d = get_dive_by_uniq_id(m_diveId);
 	if (!d)
 		return;
 	if (verbose)
-		qDebug() << "setDiveId(" << d->number << ")";
+		qDebug() << "update profile for dive #" << d->number << "offeset" << QString::number(m_xOffset, 'f', 1) << "/" << QString::number(m_yOffset, 'f', 1);
 	m_profileWidget->plotDive(d, true);
 }
 
+void QMLProfile::setDiveId(int diveId)
+{
+	m_diveId = diveId;
+	if (m_diveId < 0)
+		return;
+	updateProfile();
+}
+
 qreal QMLProfile::devicePixelRatio() const
 {
 	return m_devicePixelRatio;
@@ -107,12 +154,30 @@ void QMLProfile::setDevicePixelRatio(qreal dpr)
 {
 	if (dpr != m_devicePixelRatio) {
 		m_devicePixelRatio = dpr;
-		m_profileWidget->setFontPrintScale(0.8 * dpr);
+		m_profileWidget->setFontPrintScale(fontScale * dpr);
 		updateDevicePixelRatio(dpr);
 		emit devicePixelRatioChanged();
 	}
 }
 
+// don't update the profile here, have the user update x and y and then manually trigger an update
+void QMLProfile::setXOffset(qreal value)
+{
+	if (IS_FP_SAME(value, m_xOffset))
+		return;
+	m_xOffset = value;
+	emit xOffsetChanged();
+}
+
+// don't update the profile here, have the user update x and y and then manually trigger an update
+void QMLProfile::setYOffset(qreal value)
+{
+	if (IS_FP_SAME(value, m_yOffset))
+		return;
+	m_yOffset = value;
+	emit yOffsetChanged();
+}
+
 void QMLProfile::screenChanged(QScreen *screen)
 {
 	setDevicePixelRatio(screen->devicePixelRatio());
