@@ -5,14 +5,12 @@
 #include "core/profile.h"
 #include <QPen>
 
+static const qreal height = 3.0;
+
 TankItem::TankItem(QObject *parent) :
 	QObject(parent),
-	QGraphicsRectItem(),
-	dataModel(0),
-	pInfoEntry(0),
-	pInfoNr(0)
+	plotEndTime(-1)
 {
-	height = 3;
 	QColor red(PERSIANRED1);
 	QColor blue(AIR_BLUE);
 	QColor yellow(NITROX_YELLOW);
@@ -36,20 +34,27 @@ TankItem::TankItem(QObject *parent) :
 
 void TankItem::setData(DivePlotDataModel *model, struct plot_info *plotInfo, struct dive *d)
 {
-	free(pInfoEntry);
-	// the plotInfo and dive structures passed in could become invalid before we stop using them,
-	// so copy the data that we need
-	int size = plotInfo->nr * sizeof(plotInfo->entry[0]);
-	pInfoEntry = (struct plot_data *)malloc(size);
-	pInfoNr = plotInfo->nr;
-	memcpy(pInfoEntry, plotInfo->entry, size);
-	dataModel = model;
-	connect(dataModel, SIGNAL(dataChanged(QModelIndex, QModelIndex)), this, SLOT(modelDataChanged(QModelIndex, QModelIndex)), Qt::UniqueConnection);
+	// If there is nothing to plot, quit early.
+	if (plotInfo->nr <= 0) {
+		plotEndTime = -1;
+		return;
+	}
+
+	// Find correct end of the dive plot for correct end of the tankbar.
+	struct plot_data *last_entry = &plotInfo->entry[plotInfo->nr - 1];
+	plotEndTime = last_entry->sec;
+
+	// Stay informed of changes to the tanks.
+	connect(model, SIGNAL(dataChanged(QModelIndex, QModelIndex)), this, SLOT(modelDataChanged(QModelIndex, QModelIndex)), Qt::UniqueConnection);
+
 	modelDataChanged();
 }
 
-void TankItem::createBar(qreal x, qreal w, struct gasmix gas)
+void TankItem::createBar(int startTime, int stopTime, struct gasmix gas)
 {
+	qreal x = hAxis->posAtValue(startTime);
+	qreal w = hAxis->posAtValue(stopTime) - hAxis->posAtValue(startTime);
+
 	// pick the right gradient, size, position and text
 	QGraphicsRectItem *rect = new QGraphicsRectItem(x, 0, w, height, this);
 	if (gasmix_is_air(gas))
@@ -76,39 +81,34 @@ void TankItem::createBar(qreal x, qreal w, struct gasmix gas)
 void TankItem::modelDataChanged(const QModelIndex&, const QModelIndex&)
 {
 	// We don't have enougth data to calculate things, quit.
-	if (!dataModel || !pInfoEntry || !pInfoNr)
+	if (plotEndTime < 0)
 		return;
 
 	// remove the old rectangles
 	qDeleteAll(rects);
 	rects.clear();
 
-	qreal width, left;
-
-	// Find correct end of the dive plot for correct end of the tankbar
-	struct plot_data *last_entry = &pInfoEntry[pInfoNr-1];
+	// Bail if there are no cylinders
+	if (displayed_dive.cylinders.nr <= 0)
+		return;
 
 	// get the information directly from the displayed_dive (the dc always exists)
 	struct divecomputer *dc = get_dive_dc(&displayed_dive, dc_number);
 
 	// start with the first gasmix and at the start of the dive
 	int cyl = explicit_first_cylinder(&displayed_dive, dc);
-	struct gasmix gasmix = displayed_dive.cylinder[cyl].gasmix;
+	struct gasmix gasmix = get_cylinder(&displayed_dive, cyl)->gasmix;
 	int startTime = 0;
 
 	// work through all the gas changes and add the rectangle for each gas while it was used
 	const struct event *ev = get_next_event(dc->events, "gaschange");
-	while (ev && (int)ev->time.seconds < last_entry->sec) {
-		width = hAxis->posAtValue(ev->time.seconds) - hAxis->posAtValue(startTime);
-		left = hAxis->posAtValue(startTime);
-		createBar(left, width, gasmix);
+	while (ev && (int)ev->time.seconds < plotEndTime) {
+		createBar(startTime, ev->time.seconds, gasmix);
 		startTime = ev->time.seconds;
 		gasmix = get_gasmix_from_event(&displayed_dive, ev);
 		ev = get_next_event(ev->next, "gaschange");
 	}
-	width = hAxis->posAtValue(last_entry->sec) - hAxis->posAtValue(startTime);
-	left = hAxis->posAtValue(startTime);
-	createBar(left, width, gasmix);
+	createBar(startTime, plotEndTime, gasmix);
 }
 
 void TankItem::setHorizontalAxis(DiveCartesianAxis *horizontal)
