@@ -9,9 +9,9 @@
 #include "core/qthelper.h"
 #include "core/settings/qPrefDivePlanner.h"
 #include "core/settings/qPrefUnit.h"
-#if not defined(SUBSURFACE_MOBILE) && not defined(SUBSURFACE_TESTING)
+#if !defined(SUBSURFACE_TESTING)
 #include "commands/command.h"
-#endif // SUBSURFACE_MOBILE SUBSURFACE_TESTING
+#endif // !SUBSURFACE_TESTING
 #include "core/gettextfromc.h"
 #include "core/deco.h"
 #include <QApplication>
@@ -46,6 +46,16 @@ void DivePlannerPointsModel::removeSelectedPoints(const QVector<int> &rows)
 
 void DivePlannerPointsModel::createSimpleDive()
 {
+	// clean out the dive and give it an id and the correct dc model
+	clear_dive(&displayed_dive);
+	displayed_dive.id = dive_getUniqID();
+	displayed_dive.when = QDateTime::currentMSecsSinceEpoch() / 1000L + gettimezoneoffset() + 3600;
+	displayed_dive.dc.model = strdup("planned dive"); // don't translate! this is stored in the XML file
+
+	clear();
+	setupCylinders();
+	setupStartTime();
+
 	// initialize the start time in the plan
 	diveplan.when = displayed_dive.when;
 
@@ -75,7 +85,7 @@ void DivePlannerPointsModel::setupStartTime()
 	if (dive_table.nr) {
 		struct dive *d = get_dive(dive_table.nr - 1);
 		time_t ends = dive_endtime(d);
-		time_t diff = ends - startTime.toTime_t();
+		time_t diff = ends - dateTimeToTimestamp(startTime);
 		if (diff > 0) {
 			startTime = startTime.addSecs(diff + 3600);
 		}
@@ -423,6 +433,9 @@ DivePlannerPointsModel::DivePlannerPointsModel(QObject *parent) : QAbstractTable
 {
 	memset(&diveplan, 0, sizeof(diveplan));
 	startTime.setTimeSpec(Qt::UTC);
+	// use a Qt-connection to send the variations text across thread boundary (in case we
+	// are calculating the variations in a background thread).
+	connect(this, &DivePlannerPointsModel::variationsComputed, this, &DivePlannerPointsModel::computeVariationsDone);
 }
 
 DivePlannerPointsModel *DivePlannerPointsModel::instance()
@@ -687,7 +700,7 @@ void DivePlannerPointsModel::setSurfaceSegment(int duration)
 void DivePlannerPointsModel::setStartDate(const QDate &date)
 {
 	startTime.setDate(date);
-	diveplan.when = startTime.toTime_t();
+	diveplan.when = dateTimeToTimestamp(startTime);
 	displayed_dive.when = diveplan.when;
 	emitDataChanged();
 }
@@ -695,7 +708,7 @@ void DivePlannerPointsModel::setStartDate(const QDate &date)
 void DivePlannerPointsModel::setStartTime(const QTime &t)
 {
 	startTime.setTime(t);
-		diveplan.when = startTime.toTime_t();
+		diveplan.when = dateTimeToTimestamp(startTime);
 	displayed_dive.when = diveplan.when;
 	emitDataChanged();
 }
@@ -982,7 +995,7 @@ void DivePlannerPointsModel::createTemporaryPlan()
 		computeVariations(plan_copy, &plan_deco_state);
 #endif
 		final_deco_state = plan_deco_state;
-		emit calculatedPlanNotes();
+		emit calculatedPlanNotes(QString(displayed_dive.notes));
 	}
 	// throw away the cache
 	free(cache);
@@ -1147,6 +1160,7 @@ void DivePlannerPointsModel::computeVariations(struct diveplan *original_plan, c
 			FRACTION(analyzeVariations(shallower, original, deeper, qPrintable(depth_units)), 60), qPrintable(depth_units),
 			FRACTION(analyzeVariations(shorter, original, longer, qPrintable(time_units)), 60));
 
+		// By using a signal, we can transport the variations to the main thread.
 		emit variationsComputed(QString(buf));
 #ifdef DEBUG_STOPVAR
 		printf("\n\n");
@@ -1161,6 +1175,14 @@ finish:
 //	setRecalc(oldRecalc);
 }
 
+void DivePlannerPointsModel::computeVariationsDone(QString variations)
+{
+	QString notes = QString(displayed_dive.notes);
+	free(displayed_dive.notes);
+	displayed_dive.notes = copy_qstring(notes.replace("VARIATIONS", variations));
+	emit calculatedPlanNotes(QString(displayed_dive.notes));
+}
+
 void DivePlannerPointsModel::createPlan(bool replanCopy)
 {
 	// Ok, so, here the diveplan creates a dive
@@ -1222,22 +1244,22 @@ void DivePlannerPointsModel::createPlan(bool replanCopy)
 	if (!current_dive || displayed_dive.id != current_dive->id) {
 		// we were planning a new dive, not re-planning an existing one
 		displayed_dive.divetrip = nullptr; // Should not be necessary, just in case!
-#if not defined(SUBSURFACE_MOBILE) && not defined(SUBSURFACE_TESTING)
+#if !defined(SUBSURFACE_TESTING)
 		Command::addDive(&displayed_dive, autogroup, true);
-#endif // SUBSURFACE_MOBILE SUBSURFACE_TESTING
+#endif // !SUBSURFACE_TESTING
 	} else {
 		copy_events_until(current_dive, &displayed_dive, preserved_until.seconds);
 		if (replanCopy) {
 			// we were planning an old dive and save as a new dive
 			displayed_dive.id = dive_getUniqID(); // Things will break horribly if we create dives with the same id.
-#if not defined(SUBSURFACE_MOBILE) && not defined(SUBSURFACE_TESTING)
+#if !defined(SUBSURFACE_TESTING)
 			Command::addDive(&displayed_dive, false, false);
-#endif // SUBSURFACE_MOBILE SUBSURFACE_TESTING
+#endif // !SUBSURFACE_TESTING
 		} else {
 			// we were planning an old dive and rewrite the plan
-#if not defined(SUBSURFACE_MOBILE) && not defined(SUBSURFACE_TESTING)
+#if !defined(SUBSURFACE_TESTING)
 			Command::replanDive(&displayed_dive);
-#endif // SUBSURFACE_MOBILE SUBSURFACE_TESTING
+#endif // !SUBSURFACE_TESTING
 		}
 	}
 
