@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: GPL-2.0
 /*
  * An .slg file is composed of a main table (Dives), a bunch of tables directly
- * linked to Dives by their indexes (Site, Suit, Tank, etc) and another group of
+ * linked to Dives by their indices (Site, Suit, Tank, etc) and another group of
  * independent tables (Activity, Type, Gear, Fish ...) which connect with the dives
  * via a related group of tables (ActivityRelation, TypeRelation ...) that contain
  * just the dive index number and the related table index number.
@@ -507,7 +507,7 @@ static void merge_cylinder_info(cylinder_t *src, cylinder_t *dst)
 static int smtk_clean_cylinders(struct dive *d)
 {
 	int i = tanks - 1;
-	cylinder_t  *cyl, *base = &d->cylinder[0];
+	cylinder_t  *cyl, *base = get_cylinder(d, 0);
 
 	cyl = base + tanks - 1;
 	while (cyl != base) {
@@ -727,6 +727,8 @@ static void smtk_parse_relations(MdbHandle *mdb, struct dive *dive, char *dive_i
 
 	/* Get the text associated with the relations */
 	for (d_runner = diverel_head; d_runner; d_runner = d_runner->next) {
+		if (!list[d_runner->idx - 1])
+			continue;
 		if (tag)
 			taglist_add_tag(&dive->tag_list, list[d_runner->idx - 1]);
 		else
@@ -966,8 +968,6 @@ void smartrak_import(const char *file, struct dive_table *divetable)
 	smtk_version = atoi(smtk_ver[0]);
 	tanks = (smtk_version < 10213) ? 3 : 10;
 
-
-
 	mdb_table = smtk_open_table(mdb, "Dives", col, bound_values);
 	if (!mdb_table) {
 		report_error("[Error][smartrak_import]\tFile %s does not seem to be an SmartTrak file.", file);
@@ -1038,24 +1038,27 @@ void smartrak_import(const char *file, struct dive_table *divetable)
 		int tankidxcol = coln(TANKIDX);
 
 		for (i = 0; i < tanks; i++) {
-			if (smtkdive->cylinder[i].start.mbar == 0)
-				smtkdive->cylinder[i].start.mbar = lrint(strtod(col[(i * 2) + pstartcol]->bind_ptr, NULL) * 1000);
+			cylinder_t *tmptank = get_or_create_cylinder(smtkdive, i);
+			if (!tmptank)
+				break;
+			if (tmptank->start.mbar == 0)
+				tmptank->start.mbar = lrint(strtod(col[(i * 2) + pstartcol]->bind_ptr, NULL) * 1000);
 			/*
 			 * If there is a start pressure ensure that end pressure is not zero as
 			 * will be registered in DCs which only keep track of differential pressures,
 			 * and collect the data registered  by the user in mdb
 			 */
-			if (smtkdive->cylinder[i].end.mbar == 0 && smtkdive->cylinder[i].start.mbar != 0)
-					smtkdive->cylinder[i].end.mbar = lrint(strtod(col[(i * 2) + 1 + pstartcol]->bind_ptr, NULL) * 1000 ? : 1000);
-			if (smtkdive->cylinder[i].gasmix.o2.permille == 0)
-				smtkdive->cylinder[i].gasmix.o2.permille = lrint(strtod(col[i + o2fraccol]->bind_ptr, NULL) * 10);
+			if (tmptank->end.mbar == 0 && tmptank->start.mbar != 0)
+				tmptank->end.mbar = lrint(strtod(col[(i * 2) + 1 + pstartcol]->bind_ptr, NULL) * 1000 ? : 1000);
+			if (tmptank->gasmix.o2.permille == 0)
+				tmptank->gasmix.o2.permille = lrint(strtod(col[i + o2fraccol]->bind_ptr, NULL) * 10);
 			if (smtk_version == 10213) {
-				if (smtkdive->cylinder[i].gasmix.he.permille == 0)
-					smtkdive->cylinder[i].gasmix.he.permille = lrint(strtod(col[i + hefraccol]->bind_ptr, NULL) * 10);
+				if (tmptank->gasmix.he.permille == 0)
+					tmptank->gasmix.he.permille = lrint(strtod(col[i + hefraccol]->bind_ptr, NULL) * 10);
 			} else {
-				smtkdive->cylinder[i].gasmix.he.permille = 0;
+				tmptank->gasmix.he.permille = 0;
 			}
-			smtk_build_tank_info(mdb_clon, &smtkdive->cylinder[i], col[i + tankidxcol]->bind_ptr);
+			smtk_build_tank_info(mdb_clon, tmptank, col[i + tankidxcol]->bind_ptr);
 		}
 		/* Check for duplicated cylinders and clean them */
 		smtk_clean_cylinders(smtkdive);
