@@ -306,7 +306,7 @@ void TestParse::testParseMerge()
 int TestParse::parseCSVmanual(int units, std::string file)
 {
 	verbose = 1;
-	char *params[55];
+	char *params[59];
 	int pnr = 0;
 
 	// Numbers are column numbers
@@ -318,40 +318,43 @@ int TestParse::parseCSVmanual(int units, std::string file)
 	params[pnr++] = intdup(2);
 	params[pnr++] = strdup("durationField");
 	params[pnr++] = intdup(3);
+	// 4 Will be SAC, once we add support for reading it
 	params[pnr++] = strdup("maxDepthField");
-	params[pnr++] = intdup(4);
-	params[pnr++] = strdup("meanDepthField");
 	params[pnr++] = intdup(5);
-	params[pnr++] = strdup("airtempField");
+	params[pnr++] = strdup("meanDepthField");
 	params[pnr++] = intdup(6);
-	params[pnr++] = strdup("watertempField");
+	params[pnr++] = strdup("modeField");
 	params[pnr++] = intdup(7);
-	params[pnr++] = strdup("cylindersizeField");
+	params[pnr++] = strdup("airtempField");
 	params[pnr++] = intdup(8);
-	params[pnr++] = strdup("startpressureField");
+	params[pnr++] = strdup("watertempField");
 	params[pnr++] = intdup(9);
-	params[pnr++] = strdup("endpressureField");
+	params[pnr++] = strdup("cylindersizeField");
 	params[pnr++] = intdup(10);
-	params[pnr++] = strdup("o2Field");
+	params[pnr++] = strdup("startpressureField");
 	params[pnr++] = intdup(11);
-	params[pnr++] = strdup("heField");
+	params[pnr++] = strdup("endpressureField");
 	params[pnr++] = intdup(12);
-	params[pnr++] = strdup("locationField");
+	params[pnr++] = strdup("o2Field");
 	params[pnr++] = intdup(13);
-	params[pnr++] = strdup("gpsField");
+	params[pnr++] = strdup("heField");
 	params[pnr++] = intdup(14);
-	params[pnr++] = strdup("divemasterField");
+	params[pnr++] = strdup("locationField");
 	params[pnr++] = intdup(15);
-	params[pnr++] = strdup("buddyField");
+	params[pnr++] = strdup("gpsField");
 	params[pnr++] = intdup(16);
-	params[pnr++] = strdup("suitField");
+	params[pnr++] = strdup("divemasterField");
 	params[pnr++] = intdup(17);
+	params[pnr++] = strdup("buddyField");
+	params[pnr++] = intdup(18);
+	params[pnr++] = strdup("suitField");
+	params[pnr++] = intdup(19);
 	params[pnr++] = strdup("notesField");
-	params[pnr++] = intdup(20);
-	params[pnr++] = strdup("weightField");
-	params[pnr++] = intdup(21);
-	params[pnr++] = strdup("tagsField");
 	params[pnr++] = intdup(22);
+	params[pnr++] = strdup("weightField");
+	params[pnr++] = intdup(23);
+	params[pnr++] = strdup("tagsField");
+	params[pnr++] = intdup(24);
 	// Numbers are indices of possible options
 	params[pnr++] = strdup("separatorIndex");
 	params[pnr++] = intdup(0);
@@ -368,14 +371,29 @@ int TestParse::parseCSVmanual(int units, std::string file)
 
 void TestParse::exportCSVDiveDetails()
 {
-	parse_file(SUBSURFACE_TEST_DATA "/dives/test40.xml", &dive_table, &trip_table, &dive_site_table);
+	int saved_sac = 0;
+
+	parse_file(SUBSURFACE_TEST_DATA "/dives/test25.xml", &dive_table, &trip_table, &dive_site_table);
 
 	export_dives_xslt("testcsvexportmanual.csv", 0, 0, "xml2manualcsv.xslt", false);
 	export_dives_xslt("testcsvexportmanualimperial.csv", 0, 1, "xml2manualcsv.xslt", false);
 
+	if (dive_table.nr > 0) {
+		struct dive *dive = dive_table.dives[dive_table.nr - 1];
+		saved_sac = dive->sac;
+	}
 	clear_dive_file_data();
 
 	parseCSVmanual(1, "testcsvexportmanualimperial.csv");
+
+	// We do not currently support reading SAC, thus faking it
+	if (dive_table.nr > 0) {
+		struct dive *dive = dive_table.dives[dive_table.nr - 1];
+		dive->sac = saved_sac;
+	}
+
+
+
 	export_dives_xslt("testcsvexportmanual2.csv", 0, 0, "xml2manualcsv.xslt", false);
 
 	FILE_COMPARE("testcsvexportmanual2.csv",
@@ -384,6 +402,46 @@ void TestParse::exportCSVDiveDetails()
 	clear_dive_file_data();
 }
 
+void TestParse::exportSubsurfaceCSV()
+{
+	int saved_sac = 0;
+	char *params[59];
+	int pnr = 0;
+
+	/* Test SubsurfaceCSV with multiple cylinders */
+	parse_file(SUBSURFACE_TEST_DATA "/dives/test40.xml", &dive_table, &trip_table, &dive_site_table);
+
+	export_dives_xslt("testcsvexportmanual-cyl.csv", 0, 0, "xml2manualcsv.xslt", false);
+	export_dives_xslt("testcsvexportmanualimperial-cyl.csv", 0, 1, "xml2manualcsv.xslt", false);
+
+	if (dive_table.nr > 0) {
+		struct dive *dive = dive_table.dives[dive_table.nr - 1];
+		saved_sac = dive->sac;
+	}
+
+	clear_dive_file_data();
+
+	params[pnr++] = strdup("separatorIndex");
+	params[pnr++] = intdup(0);
+	params[pnr++] = strdup("units");
+	params[pnr++] = intdup(1);
+	params[pnr++] = 0;
+	parse_csv_file("testcsvexportmanualimperial-cyl.csv", params, pnr - 1, "SubsurfaceCSV", &dive_table, &trip_table, &dive_site_table);
+
+	// We do not currently support reading SAC, thus faking it
+	if (dive_table.nr > 0) {
+		struct dive *dive = dive_table.dives[dive_table.nr - 1];
+		dive->sac = saved_sac;
+	}
+
+	export_dives_xslt("testcsvexportmanual2-cyl.csv", 0, 0, "xml2manualcsv.xslt", false);
+
+	FILE_COMPARE("testcsvexportmanual2-cyl.csv",
+		     "testcsvexportmanual-cyl.csv");
+
+	clear_dive_file_data();
+}
+
 int TestParse::parseCSVprofile(int units, std::string file)
 {
 	verbose = 1;
@@ -453,6 +511,7 @@ void TestParse::exportUDDF()
 void TestParse::testExport()
 {
 	exportCSVDiveDetails();
+	exportSubsurfaceCSV();
 	exportCSVDiveProfile();
 	exportUDDF();
 }
